"""
Module contains tools for collecting data from various remote sources
"""

import warnings

from pandas_datareader.av.forex import AVForexReader
from pandas_datareader.av.quotes import AVQuotesReader
from pandas_datareader.av.sector import AVSectorPerformanceReader
from pandas_datareader.av.time_series import AVTimeSeriesReader
from pandas_datareader.bankofcanada import BankOfCanadaReader
from pandas_datareader.enigma import EnigmaReader
from pandas_datareader.eurostat import EurostatReader
from pandas_datareader.exceptions import DEP_ERROR_MSG, \
    ImmediateDeprecationError
from pandas_datareader.famafrench import FamaFrenchReader
from pandas_datareader.fred import FredReader
from pandas_datareader.google.daily import GoogleDailyReader
from pandas_datareader.google.options import Options as GoogleOptions
from pandas_datareader.google.quotes import GoogleQuotesReader
from pandas_datareader.iex.daily import IEXDailyReader
from pandas_datareader.iex.deep import Deep as IEXDeep
from pandas_datareader.iex.tops import LastReader as IEXLasts, \
    TopsReader as IEXTops
from pandas_datareader.moex import MoexReader
from pandas_datareader.mstar.daily import MorningstarDailyReader
from pandas_datareader.nasdaq_trader import get_nasdaq_symbols
from pandas_datareader.oecd import OECDReader
from pandas_datareader.quandl import QuandlReader
from pandas_datareader.robinhood import RobinhoodHistoricalReader, \
    RobinhoodQuoteReader
from pandas_datareader.stooq import StooqDailyReader
from pandas_datareader.tiingo import TiingoDailyReader, TiingoQuoteReader
from pandas_datareader.yahoo.actions import (YahooActionReader, YahooDivReader)
from pandas_datareader.yahoo.components import _get_data as \
    get_components_yahoo
from pandas_datareader.yahoo.daily import YahooDailyReader
from pandas_datareader.yahoo.options import Options as YahooOptions
from pandas_datareader.yahoo.quotes import YahooQuotesReader

__all__ = ['get_components_yahoo', 'get_data_enigma', 'get_data_famafrench',
           'get_data_fred', 'get_data_google', 'get_data_moex',
           'get_data_quandl', 'get_data_yahoo', 'get_data_yahoo_actions',
           'get_nasdaq_symbols', 'get_quote_google', 'get_quote_yahoo',
           'get_tops_iex', 'get_summary_iex', 'get_records_iex',
           'get_recent_iex', 'get_markets_iex', 'get_last_iex',
           'get_iex_symbols', 'get_iex_book', 'get_dailysummary_iex',
           'get_data_morningstar', 'get_data_stooq',
           'get_data_stooq', 'get_data_robinhood', 'get_quotes_robinhood',
           'DataReader']


def get_data_alphavantage(*args, **kwargs):
    return AVTimeSeriesReader(*args, **kwargs).read()


def get_data_fred(*args, **kwargs):
    return FredReader(*args, **kwargs).read()


def get_data_famafrench(*args, **kwargs):
    return FamaFrenchReader(*args, **kwargs).read()


def get_data_google(*args, **kwargs):
    return GoogleDailyReader(*args, **kwargs).read()


def get_data_yahoo(*args, **kwargs):
    return YahooDailyReader(*args, **kwargs).read()


def get_data_enigma(*args, **kwargs):
    return EnigmaReader(*args, **kwargs).read()


def get_quote_av(*args, **kwargs):
    return AVQuotesReader(*args, **kwargs).read()


def get_data_yahoo_actions(*args, **kwargs):
    return YahooActionReader(*args, **kwargs).read()


def get_quote_yahoo(*args, **kwargs):
    return YahooQuotesReader(*args, **kwargs).read()


def get_quote_google(*args, **kwargs):
    raise ImmediateDeprecationError(DEP_ERROR_MSG.format('Google Quotes'))
    return GoogleQuotesReader(*args, **kwargs).read()


def get_data_quandl(*args, **kwargs):
    return QuandlReader(*args, **kwargs).read()


def get_data_moex(*args, **kwargs):
    return MoexReader(*args, **kwargs).read()


def get_data_stooq(*args, **kwargs):
    return StooqDailyReader(*args, **kwargs).read()


def get_tops_iex(*args, **kwargs):
    return IEXTops(*args, **kwargs).read()


def get_last_iex(*args, **kwargs):
    return IEXLasts(*args, **kwargs).read()


def get_data_morningstar(*args, **kwargs):
    return MorningstarDailyReader(*args, **kwargs).read()


def get_data_robinhood(*args, **kwargs):
    return RobinhoodHistoricalReader(*args, **kwargs).read()


def get_quotes_robinhood(*args, **kwargs):
    return RobinhoodQuoteReader(*args, **kwargs).read()


def get_data_tiingo(*args, **kwargs):
    return TiingoDailyReader(*args, **kwargs).read()


def get_quotes_tiingo(*args, **kwargs):
    return TiingoQuoteReader(*args, **kwargs).read()


def get_exchange_rate_av(*args, **kwargs):
    return AVForexReader(*args, **kwargs).read()


def get_sector_performance_av(*args, **kwargs):
    return AVSectorPerformanceReader(*args, **kwargs).read()


def get_markets_iex(*args, **kwargs):
    """
    Returns near-real time volume data across markets segregated by tape
    and including a percentage of overall volume during the session

    This endpoint does not accept any parameters.

    Reference: https://www.iextrading.com/developer/docs/#markets

    :return: DataFrame
    """
    from pandas_datareader.iex.market import MarketReader
    return MarketReader(*args, **kwargs).read()


def get_dailysummary_iex(*args, **kwargs):
    """
    Returns a summary of daily market volume statistics. Without parameters,
    this will return the most recent trading session by default.

    :param start:
        A datetime object - the beginning of the date range.
    :param end:
        A datetime object - the end of the date range.

    Reference: https://www.iextrading.com/developer/docs/#historical-daily

    :return: DataFrame
    """
    from pandas_datareader.iex.stats import DailySummaryReader
    return DailySummaryReader(*args, **kwargs).read()


def get_summary_iex(*args, **kwargs):
    """
    Returns an aggregated monthly summary of market volume and a variety of
    related metrics for trades by lot size, security market cap, and venue.
    In the absence of parameters, this will return month-to-date statistics.
    For ranges spanning multiple months, this will return one row per month.

    :param start:
        A datetime object - the beginning of the date range.
    :param end:
        A datetime object - the end of the date range.

    :return: DataFrame
    """
    from pandas_datareader.iex.stats import MonthlySummaryReader
    return MonthlySummaryReader(*args, **kwargs).read()


def get_records_iex(*args, **kwargs):
    """
    Returns the record value, record date, recent value, and 30-day average for
    market volume, # of symbols traded, # of routed trades and notional value.
    This function accepts no additional parameters.

    Reference: https://www.iextrading.com/developer/docs/#records

    :return: DataFrame
    """
    from pandas_datareader.iex.stats import RecordsReader
    return RecordsReader(*args, **kwargs).read()


def get_recent_iex(*args, **kwargs):
    """
    Returns market volume and trade routing statistics for recent sessions.
    Also reports IEX's relative market share, lit share volume and a boolean
    halfday indicator.

    Reference: https://www.iextrading.com/developer/docs/#recent

    :return: DataFrame
    """
    from pandas_datareader.iex.stats import RecentReader
    return RecentReader(*args, **kwargs).read()


def get_iex_symbols(*args, **kwargs):
    """
    Returns a list of all equity symbols available for trading on IEX. Accepts
    no additional parameters.

    Reference: https://www.iextrading.com/developer/docs/#symbols

    :return: DataFrame
    """
    from pandas_datareader.iex.ref import SymbolsReader
    return SymbolsReader(*args, **kwargs).read()


def get_iex_book(*args, **kwargs):
    """
    Returns an array of dictionaries with depth of book data from IEX for up to
    10 securities at a time. Returns a dictionary of the bid and ask books.

    :param symbols:
        A string or list of strings of valid tickers
    :param service:
        'book': Live depth of book data
        'op-halt-status': Checks to see if the exchange has instituted a halt
        'security-event': Denotes individual security related event
        'ssr-status': Short Sale Price Test restrictions, per reg 201 of SHO
        'system-event': Relays current feed status (i.e. market open)
        'trades': Retrieves recent executions, trade size/price and flags
        'trade-breaks': Lists execution breaks for the current trading session
        'trading-status': Returns status and cause codes for securities

    :return: Object
    """
    return IEXDeep(*args, **kwargs).read()


def DataReader(name, data_source=None, start=None, end=None,
               retry_count=3, pause=0.1, session=None, access_key=None):
    """
    Imports data from a number of online sources.

    Currently supports Google Finance, St. Louis FED (FRED),
    and Kenneth French's data library, among others.

    Parameters
    ----------
    name : str or list of strs
        the name of the dataset. Some data sources (google, fred) will
        accept a list of names.
    data_source: {str, None}
        the data source ("google", "fred", "ff")
    start : {datetime, None}
        left boundary for range (defaults to 1/1/2010)
    end : {datetime, None}
        right boundary for range (defaults to today)
    retry_count : {int, 3}
        Number of times to retry query request.
    pause : {numeric, 0.001}
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    access_key : (str, None)
        Optional parameter to specify an API key for certain data sources.

    Examples
    ----------
    # Data from Google Finance
    aapl = DataReader("AAPL", "google")

    # Price and volume data from IEX
    tops = DataReader(["GS", "AAPL"], "iex-tops")
    # Top of book executions from IEX
    gs = DataReader("GS", "iex-last")
    # Real-time depth of book data from IEX
    gs = DataReader("GS", "iex-book")

    # Data from FRED
    vix = DataReader("VIXCLS", "fred")

    # Data from Fama/French
    ff = DataReader("F-F_Research_Data_Factors", "famafrench")
    ff = DataReader("F-F_Research_Data_Factors_weekly", "famafrench")
    ff = DataReader("6_Portfolios_2x3", "famafrench")
    ff = DataReader("F-F_ST_Reversal_Factor", "famafrench")
    """
    if data_source == "yahoo":
        return YahooDailyReader(symbols=name, start=start, end=end,
                                adjust_price=False, chunksize=25,
                                retry_count=retry_count, pause=pause,
                                session=session).read()

    elif data_source == "google":
        return GoogleDailyReader(symbols=name, start=start, end=end,
                                 chunksize=25,
                                 retry_count=retry_count, pause=pause,
                                 session=session).read()

    elif data_source == "iex":
        return IEXDailyReader(symbols=name, start=start, end=end,
                              chunksize=25,
                              retry_count=retry_count, pause=pause,
                              session=session).read()

    elif data_source == "iex-tops":
        return IEXTops(symbols=name, start=start, end=end,
                       retry_count=retry_count, pause=pause,
                       session=session).read()

    elif data_source == "iex-last":
        return IEXLasts(symbols=name, start=start, end=end,
                        retry_count=retry_count, pause=pause,
                        session=session).read()

    elif data_source == "bankofcanada":
        return BankOfCanadaReader(symbols=name, start=start, end=end,
                                  retry_count=retry_count, pause=pause,
                                  session=session).read()
    elif data_source == "stooq":
        return StooqDailyReader(symbols=name,
                                chunksize=25,
                                retry_count=retry_count, pause=pause,
                                session=session).read()

    elif data_source == "iex-book":
        return IEXDeep(symbols=name, service="book", start=start, end=end,
                       retry_count=retry_count, pause=pause,
                       session=session).read()

    elif data_source == "enigma":
        return EnigmaReader(dataset_id=name, api_key=access_key).read()

    elif data_source == "fred":
        return FredReader(symbols=name, start=start, end=end,
                          retry_count=retry_count, pause=pause,
                          session=session).read()

    elif data_source == "famafrench":
        return FamaFrenchReader(symbols=name, start=start, end=end,
                                retry_count=retry_count, pause=pause,
                                session=session).read()

    elif data_source == "oecd":
        return OECDReader(symbols=name, start=start, end=end,
                          retry_count=retry_count, pause=pause,
                          session=session).read()
    elif data_source == "eurostat":
        return EurostatReader(symbols=name, start=start, end=end,
                              retry_count=retry_count, pause=pause,
                              session=session).read()
    elif data_source == 'nasdaq':
        if name != 'symbols':
            raise ValueError("Only the string 'symbols' is supported for "
                             "Nasdaq, not %r" % (name,))
        return get_nasdaq_symbols(retry_count=retry_count, pause=pause)

    elif data_source == "quandl":
        return QuandlReader(symbols=name, start=start, end=end,
                            retry_count=retry_count, pause=pause,
                            session=session, api_key=access_key).read()
    elif data_source == "moex":
        return MoexReader(symbols=name, start=start, end=end,
                          retry_count=retry_count, pause=pause,
                          session=session).read()
    elif data_source == "morningstar":
        return MorningstarDailyReader(symbols=name, start=start, end=end,
                                      retry_count=retry_count, pause=pause,
                                      session=session, interval="d").read()
    elif data_source == 'robinhood':
        return RobinhoodHistoricalReader(symbols=name, start=start, end=end,
                                         retry_count=retry_count, pause=pause,
                                         session=session).read()
    elif data_source == 'tiingo':
        return TiingoDailyReader(symbols=name, start=start, end=end,
                                 retry_count=retry_count, pause=pause,
                                 session=session,
                                 api_key=access_key).read()

    elif data_source == "yahoo-actions":
        return YahooActionReader(symbols=name, start=start, end=end,
                                 retry_count=retry_count, pause=pause,
                                 session=session).read()

    elif data_source == "yahoo-dividends":
        return YahooDivReader(symbols=name, start=start, end=end,
                              adjust_price=False, chunksize=25,
                              retry_count=retry_count, pause=pause,
                              session=session, interval='d').read()

    elif data_source == "av-forex":
        return AVForexReader(symbols=name, retry_count=retry_count,
                             pause=pause, session=session,
                             api_key=access_key).read()

    elif data_source == "av-daily":
        return AVTimeSeriesReader(symbols=name,
                                  function="TIME_SERIES_DAILY", start=start,
                                  end=end, retry_count=retry_count,
                                  pause=pause, session=session,
                                  api_key=access_key).read()

    elif data_source == "av-daily-adjusted":
        return AVTimeSeriesReader(symbols=name,
                                  function="TIME_SERIES_DAILY_ADJUSTED",
                                  start=start, end=end,
                                  retry_count=retry_count, pause=pause,
                                  session=session, api_key=access_key).read()

    elif data_source == "av-weekly":
        return AVTimeSeriesReader(symbols=name,
                                  function="TIME_SERIES_WEEKLY", start=start,
                                  end=end, retry_count=retry_count,
                                  pause=pause, session=session,
                                  api_key=access_key).read()

    elif data_source == "av-weekly-adjusted":
        return AVTimeSeriesReader(symbols=name,
                                  function="TIME_SERIES_WEEKLY_ADJUSTED",
                                  start=start, end=end,
                                  retry_count=retry_count, pause=pause,
                                  session=session, api_key=access_key).read()

    elif data_source == "av-monthly":
        return AVTimeSeriesReader(symbols=name,
                                  function="TIME_SERIES_MONTHLY", start=start,
                                  end=end, retry_count=retry_count,
                                  pause=pause, session=session,
                                  api_key=access_key).read()

    elif data_source == "av-monthly-adjusted":
        return AVTimeSeriesReader(symbols=name,
                                  function="TIME_SERIES_MONTHLY_ADJUSTED",
                                  start=start, end=end,
                                  retry_count=retry_count, pause=pause,
                                  session=session, api_key=access_key).read()

    else:
        msg = "data_source=%r is not implemented" % data_source
        raise NotImplementedError(msg)


def Options(symbol, data_source=None, session=None):
    if data_source is None:
        warnings.warn("Options(symbol) is deprecated, use Options(symbol,"
                      " data_source) instead", FutureWarning, stacklevel=2)
        data_source = "yahoo"
    if data_source == "yahoo":
        raise ImmediateDeprecationError(DEP_ERROR_MSG.format('Yahoo Options'))
        return YahooOptions(symbol, session=session)
    elif data_source == "google":
        raise ImmediateDeprecationError(DEP_ERROR_MSG.format('Google Options'))
        return GoogleOptions(symbol, session=session)
    else:
        raise NotImplementedError("currently only yahoo and google supported")



from datetime import date, timedelta

import pytest

import pandas_datareader.data as web
from pandas_datareader._utils import RemoteDataError


class TestBankOfCanada(object):

    @staticmethod
    def get_symbol(currency_code, inverted=False):
        if inverted:
            return 'FXCAD{}'.format(currency_code)
        else:
            return 'FX{}CAD'.format(currency_code)

    def check_bankofcanada_count(self, code):
        start, end = date.today() - timedelta(days=30), date.today()
        df = web.DataReader(self.get_symbol(code), 'bankofcanada', start, end)
        assert df.size > 15

    def check_bankofcanada_valid(self, code):
        symbol = self.get_symbol(code)
        df = web.DataReader(symbol, 'bankofcanada',
                            date.today() - timedelta(days=30), date.today())
        assert symbol in df.columns

    def check_bankofcanada_inverted(self, code):
        symbol = self.get_symbol(code)
        symbol_inverted = self.get_symbol(code, inverted=True)

        df = web.DataReader(symbol, 'bankofcanada',
                            date.today() - timedelta(days=30), date.today())
        df_i = web.DataReader(symbol_inverted, 'bankofcanada',
                              date.today() - timedelta(days=30), date.today())

        pairs = zip((1 / df)[symbol].tolist(), df_i[symbol_inverted].tolist())
        assert all(a - b < 0.01 for a, b in pairs)

    def test_bankofcanada_usd_count(self):
        self.check_bankofcanada_count('USD')

    def test_bankofcanada_eur_count(self):
        self.check_bankofcanada_count('EUR')

    def test_bankofcanada_usd_valid(self):
        self.check_bankofcanada_valid('USD')

    def test_bankofcanada_eur_valid(self):
        self.check_bankofcanada_valid('EUR')

    def test_bankofcanada_usd_inverted(self):
        self.check_bankofcanada_inverted('USD')

    def test_bankofcanada_eur_inverted(self):
        self.check_bankofcanada_inverted('EUR')

    def test_bankofcanada_bad_range(self):
        with pytest.raises(ValueError):
            web.DataReader('FXCADUSD', 'bankofcanada',
                           date.today(), date.today() - timedelta(days=30))

    def test_bankofcanada_bad_url(self):
        with pytest.raises(RemoteDataError):
            web.DataReader('abcdefgh', 'bankofcanada',
                           date.today() - timedelta(days=30), date.today())



from pandas_datareader.av import AlphaVantage

from pandas_datareader._utils import RemoteDataError

import pandas as pd


class AVForexReader(AlphaVantage):
    """
    Returns DataFrame of the Alpha Vantage Foreign Exchange (FX) Exchange Rates
    data.

    .. versionadded:: 0.7.0

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series)
        Single currency pair (formatted 'FROM/TO') or list of the same.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    api_key : str, optional
        Alpha Vantage API key . If not provided the environmental variable
        ALPHAVANTAGE_API_KEY is read. The API key is *required*.
    """
    def __init__(self, symbols=None, retry_count=3, pause=0.1, session=None,
                 api_key=None):

        super(AVForexReader, self).__init__(symbols=symbols,
                                            start=None, end=None,
                                            retry_count=retry_count,
                                            pause=pause,
                                            session=session,
                                            api_key=api_key)
        self.from_curr = {}
        self.to_curr = {}
        self.optional_params = {}
        if isinstance(symbols, str):
            self.symbols = [symbols]
        else:
            self.symbols = symbols
        try:
            for pair in self.symbols:
                self.from_curr[pair] = pair.split('/')[0]
                self.to_curr[pair] = pair.split('/')[1]
        except Exception as e:
            print(e)
            raise ValueError("Please input a currency pair "
                             "formatted 'FROM/TO' or a list of "
                             "currency symbols")

    @property
    def function(self):
        return 'CURRENCY_EXCHANGE_RATE'

    @property
    def data_key(self):
        return 'Realtime Currency Exchange Rate'

    @property
    def params(self):
        params = {
            'apikey': self.api_key,
            'function': self.function
            }
        params.update(self.optional_params)
        return params

    def read(self):
        result = []
        for pair in self.symbols:
            self.optional_params = {
                'from_currency': self.from_curr[pair],
                'to_currency': self.to_curr[pair],
                }
            data = super(AVForexReader, self).read()
            result.append(data)
        df = pd.concat(result, axis=1)
        df.columns = self.symbols
        return df

    def _read_lines(self, out):
        try:
            df = pd.DataFrame.from_dict(out[self.data_key], orient='index')
        except KeyError:
            raise RemoteDataError()
        df.sort_index(ascending=True, inplace=True)
        df.index = [id[3:] for id in df.index]
        return df



import os
import pytest

from pandas_datareader.data import get_sector_performance_av

import pandas as pd

TEST_API_KEY = os.getenv('ALPHAVANTAGE_API_KEY')
TEST_API_KEY = None if not TEST_API_KEY else TEST_API_KEY


class TestAVSector(object):

    @classmethod
    def setup_class(cls):
        pytest.importorskip('lxml')

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_sector(self):
        df = get_sector_performance_av()

        cols = pd.Index(["RT", "1D", "5D", "1M", "3M", "YTD", "1Y", "3Y", "5Y",
                         "10Y"])
        assert df.columns.equals(cols)
        assert "Energy" in df.index
        assert "Real Estate" in df.index



from pandas_datareader.av import AlphaVantage

from datetime import datetime


class AVTimeSeriesReader(AlphaVantage):
    """
    Returns DataFrame of the Alpha Vantage Stock Time Series endpoints

    .. versionadded:: 0.7.0

    Parameters
    ----------
    symbols : string
        Single stock symbol (ticker)
    start : string, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : string, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    api_key : str, optional
        AlphaVantage API key . If not provided the environmental variable
        ALPHAVANTAGE_API_KEY is read. The API key is *required*.
    """
    _FUNC_TO_DATA_KEY = {
            "TIME_SERIES_DAILY": "Time Series (Daily)",
            "TIME_SERIES_DAILY_ADJUSTED": "Time Series (Daily)",
            "TIME_SERIES_WEEKLY": "Weekly Time Series",
            "TIME_SERIES_WEEKLY_ADJUSTED": "Weekly Adjusted Time Series",
            "TIME_SERIES_MONTHLY": "Monthly Time Series",
            "TIME_SERIES_MONTHLY_ADJUSTED": "Monthly Adjusted Time Series"
    }

    def __init__(self, symbols=None, function="TIME_SERIES_DAILY",
                 start=None, end=None, retry_count=3, pause=0.1,
                 session=None, chunksize=25, api_key=None):
        super(AVTimeSeriesReader, self).__init__(symbols=symbols, start=start,
                                                 end=end,
                                                 retry_count=retry_count,
                                                 pause=pause, session=session,
                                                 api_key=api_key)

        self.func = function

    @property
    def function(self):
        return self.func

    @property
    def output_size(self):
        """ Used to limit the size of the Alpha Vantage query when
        possible.
        """
        delta = datetime.now() - self.start
        return 'full' if delta.days > 80 else 'compact'

    @property
    def data_key(self):
        return self._FUNC_TO_DATA_KEY[self.function]

    @property
    def params(self):
        return {
            "symbol": self.symbols,
            "function": self.function,
            "apikey": self.api_key,
            "outputsize": self.output_size
        }

    def _read_lines(self, out):
        data = super(AVTimeSeriesReader, self)._read_lines(out)
        start_str = self.start.strftime('%Y-%m-%d')
        end_str = self.end.strftime('%Y-%m-%d')
        data = data.loc[start_str:end_str]
        if data.empty:
            raise ValueError("Please input a valid date range")
        else:
            for column in data.columns:
                if column == 'volume':
                    data[column] = data[column].astype('int64')
                else:
                    data[column] = data[column].astype('float64')
        return data



# flake8: noqa

import datetime as dt

import pandas as pd
from pandas.compat import StringIO

from pandas_datareader.base import _DailyBaseReader
from pandas_datareader.compat import is_list_like


class MoexReader(_DailyBaseReader):
    """
    Returns a DataFrame of historical stock prices from symbols from Moex

    Parameters
    ----------
    symbols : str, an array-like object (list, tuple, Series), or a DataFrame
        A single stock symbol (secid), an array-like object of symbols or
        a DataFrame with an index containing stock symbols.
    start : str, (defaults to '1/1/2010')
        The starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : str, (defaults to today)
        The ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        The number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        The number of symbols to download consecutively before intiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used

    Notes
    -----
    To avoid being penalized by Moex servers, pauses more than 0.1s between
    downloading 'chunks' of symbols can be specified.
    """

    def __init__(self, *args, **kwargs):
        super(MoexReader, self).__init__(*args, **kwargs)
        self.start = self.start.date()
        self.end_dt = self.end
        self.end = self.end.date()

        if isinstance(self.symbols, pd.DataFrame):
            self.symbols = self.symbols.index.tolist()
        elif not is_list_like(self.symbols):
            self.symbols = [self.symbols]

        self.__engines, self.__markets = {}, {}  # dicts for engines and markets

    __url_metadata = "https://iss.moex.com/iss/securities/{symbol}.csv"
    __url_data = "https://iss.moex.com/iss/history/engines/{engine}/" \
                 "markets/{market}/securities/{symbol}.csv"

    @property
    def url(self):
        """Return a list of API URLs per symbol"""

        if not self.__engines or not self.__markets:
            raise Exception("Accessing url property before invocation "
                "of read() or _get_metadata() methods")

        return [self.__url_data.format(
                    engine=self.__engines[s],
                    market=self.__markets[s],
                    symbol=s) for s in self.symbols]

    def _get_params(self, start):
        """Return a dict for REST API GET request parameters"""

        params = {
            'iss.only': 'history',
            'iss.dp': 'point',
            'iss.df': '%Y-%m-%d',
            'iss.tf': '%H:%M:%S',
            'iss.dft': '%Y-%m-%d %H:%M:%S',
            'iss.json': 'extended',
            'callback': 'JSON_CALLBACK',
            'from': start,
            'till': self.end_dt.strftime('%Y-%m-%d'),
            'limit': 100,
            'start': 1,
            'sort_order': 'TRADEDATE',
            'sort_order_desc': 'asc'
        }
        return params

    def _get_metadata(self):
        """Get markets and engines for the given symbols"""

        markets, engines = {}, {}

        for symbol in self.symbols:
            response = self._get_response(
                self.__url_metadata.format(symbol=symbol)
            )
            text = self._sanitize_response(response)
            if len(text) == 0:
                service = self.__class__.__name__
                raise IOError("{} request returned no data; check URL for invalid "
                              "inputs: {}".format(service, self.__url_metadata))
            if isinstance(text, pd.compat.binary_type):
                text = text.decode('windows-1251')

            header_str = 'secid;boardid;'
            get_data = False
            for s in text.splitlines():
                if s.startswith(header_str):
                    get_data = True
                    continue
                if get_data and s != '':
                    fields = s.split(';')
                    markets[symbol], engines[symbol] = fields[5], fields[7]
                    break
            if symbol not in markets or symbol not in engines:
                raise IOError("{} request returned no metadata: {}\n"
                              "Typo in the security symbol `{}`?".format(
                                self.__class__.__name__,
                                self.__url_metadata.format(symbol=symbol),
                                symbol))
        return markets, engines

    def read(self):
        """Read data"""

        try:
            self.__markets, self.__engines = self._get_metadata()
            urls = self.url  # generate urls per symbols
            dfs = []  # an array of pandas dataframes per symbol to concatenate

            for i, symbol in enumerate(self.symbols):
                out_list = []
                date_column = None

                while True:  # read in a loop with small date intervals
                    if len(out_list) > 0:
                        if date_column is None:
                            date_column = out_list[0].split(';').index('TRADEDATE')

                        # get the last downloaded date
                        start_str = out_list[-1].split(';', 4)[date_column]
                        start = dt.datetime.strptime(start_str, '%Y-%m-%d').date()
                    else:
                        start_str = self.start.strftime('%Y-%m-%d')
                        start = self.start

                    if start >= self.end or start >= dt.date.today():
                        break

                    params = self._get_params(start_str)
                    strings_out = self._read_url_as_String(urls[i], params) \
                                      .splitlines()[2:]
                    strings_out = list(filter(lambda x: x.strip(), strings_out))

                    if len(out_list) == 0:
                        out_list = strings_out
                        if len(strings_out) < 101:  # all data received - break
                            break
                    else:
                        out_list += strings_out[1:]  # remove a CSV head line
                        if len(strings_out) < 100:  # all data recevied - break
                            break
                str_io = StringIO('\r\n'.join(out_list))
                dfs.append(self._read_lines(str_io)) # add a new DataFrame
        finally:
            self.close()

        if len(dfs) > 1:
            return pd.concat(dfs, axis=0, join='outer', sort=True)
        else:
            return dfs[0]

    def _read_url_as_String(self, url, params=None):
        """Open an url (and retry)"""

        response = self._get_response(url, params=params)
        text = self._sanitize_response(response)
        if len(text) == 0:
            service = self.__class__.__name__
            raise IOError("{} request returned no data; check URL for invalid "
                          "inputs: {}".format(service, self.url))
        if isinstance(text, pd.compat.binary_type):
            text = text.decode('windows-1251')
        return text

    def _read_lines(self, input):
        """Return a pandas DataFrame from input"""

        return pd.read_csv(input, index_col='TRADEDATE', parse_dates=True,
                           sep=';', na_values=('-', 'null'))



import os

from pandas_datareader.base import _BaseReader
from pandas_datareader._utils import RemoteDataError

import pandas as pd

AV_BASE_URL = 'https://www.alphavantage.co/query'


class AlphaVantage(_BaseReader):
    """
    Base class for all Alpha Vantage queries

    Notes
    -----
    See `Alpha Vantage <https://www.alphavantage.co/>`__
    """
    _format = 'json'

    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None, api_key=None):
        super(AlphaVantage, self).__init__(symbols=symbols, start=start,
                                           end=end, retry_count=retry_count,
                                           pause=pause, session=session)
        if api_key is None:
            api_key = os.getenv('ALPHAVANTAGE_API_KEY')
        if not api_key or not isinstance(api_key, str):
            raise ValueError('The AlphaVantage API key must be provided '
                             'either through the api_key variable or '
                             'through the environment varaible '
                             'ALPHAVANTAGE_API_KEY')
        self.api_key = api_key

    @property
    def url(self):
        """ API URL """
        return AV_BASE_URL

    @property
    def params(self):
        return {
            'function': self.function,
            'apikey': self.api_key
        }

    @property
    def function(self):
        """ Alpha Vantage endpoint function"""
        raise NotImplementedError

    @property
    def data_key(self):
        """ Key of data returned from Alpha Vantage """
        raise NotImplementedError

    def _read_lines(self, out):
        try:
            df = pd.DataFrame.from_dict(out[self.data_key], orient='index')
        except KeyError:
            if "Error Message" in out:
                raise ValueError("The requested symbol {} could not be "
                                 "retrived. Check valid ticker"
                                 ".".format(self.symbols))
            else:
                raise RemoteDataError()
        df = df[sorted(df.columns)]
        df.columns = [id[3:] for id in df.columns]
        return df



from __future__ import division

import json
import re
import time
from pandas import (DataFrame, to_datetime, notnull, isnull)
from pandas_datareader._utils import RemoteDataError
from pandas_datareader.base import _DailyBaseReader


class YahooDailyReader(_DailyBaseReader):
    """
    Returns DataFrame of with historical over date range,
    start to end.
    To avoid being penalized by Yahoo! Finance servers, pauses between
    downloading 'chunks' of symbols can be specified.

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    start : string, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : string, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    adjust_price : bool, default False
        If True, adjusts all prices in hist_data ('Open', 'High', 'Low',
        'Close') based on 'Adj Close' price. Adds 'Adj_Ratio' column and drops
        'Adj Close'.
    ret_index : bool, default False
        If True, includes a simple return index 'Ret_Index' in hist_data.
    chunksize : int, default 25
        Number of symbols to download consecutively before intiating pause.
    interval : string, default 'd'
        Time interval code, valid values are 'd' for daily, 'w' for weekly,
        'm' for monthly.
    get_actions : bool, default False
        If True, adds Dividend and Split columns to dataframe.
    adjust_dividends: bool, default false
        If True, adjusts dividends for splits.
    """

    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None, adjust_price=False,
                 ret_index=False, chunksize=1, interval='d',
                 get_actions=False, adjust_dividends=False):
        super(YahooDailyReader, self).__init__(symbols=symbols,
                                               start=start, end=end,
                                               retry_count=retry_count,
                                               pause=pause, session=session,
                                               chunksize=chunksize)

        # Ladder up the wait time between subsequent requests to improve
        # probability of a successful retry
        self.pause_multiplier = 2.5

        self.headers = {
            'Connection': 'keep-alive',
            'Expires': str(-1),
            'Upgrade-Insecure-Requests': str(1),
            # Google Chrome:
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2840.99 Safari/537.36'  # noqa
        }

        self.adjust_price = adjust_price
        self.ret_index = ret_index
        self.interval = interval
        self._get_actions = get_actions

        if self.interval not in ['d', 'wk', 'mo', 'm', 'w']:
            raise ValueError("Invalid interval: valid values are  'd', 'wk' and 'mo'. 'm' and 'w' have been implemented for "  # noqa
                             "backward compatibility. 'v' has been moved to the yahoo-actions or yahoo-dividends APIs.")  # noqa
        elif self.interval in ['m', 'mo']:
            self.pdinterval = 'm'
            self.interval = 'mo'
        elif self.interval in ['w', 'wk']:
            self.pdinterval = 'w'
            self.interval = 'wk'

        self.interval = '1' + self.interval
        self.adjust_dividends = adjust_dividends

    @property
    def get_actions(self):
        return self._get_actions

    @property
    def url(self):
        return 'https://finance.yahoo.com/quote/{}/history'

    # Test test_get_data_interval() crashed because of this issue, probably
    # whole yahoo part of package wasn't
    # working properly
    def _get_params(self, symbol):
        # This needed because yahoo returns data shifted by 4 hours ago.
        four_hours_in_seconds = 14400
        unix_start = int(time.mktime(self.start.timetuple()))
        unix_start += four_hours_in_seconds
        day_end = self.end.replace(hour=23, minute=59, second=59)
        unix_end = int(time.mktime(day_end.timetuple()))
        unix_end += four_hours_in_seconds

        params = {
            'period1': unix_start,
            'period2': unix_end,
            'interval': self.interval,
            'frequency': self.interval,
            'filter': 'history',
            'symbol': symbol
        }
        return params

    def _read_one_data(self, url, params):
        """ read one data from specified symbol """

        symbol = params['symbol']
        del params['symbol']
        url = url.format(symbol)

        resp = self._get_response(url, params=params)
        ptrn = r'root\.App\.main = (.*?);\n}\(this\)\);'
        try:
            j = json.loads(re.search(ptrn, resp.text, re.DOTALL).group(1))
            data = j['context']['dispatcher']['stores']['HistoricalPriceStore']
        except KeyError:
            msg = 'No data fetched for symbol {} using {}'
            raise RemoteDataError(msg.format(symbol, self.__class__.__name__))

        # price data
        prices = DataFrame(data['prices'])
        prices.columns = [col.capitalize() for col in prices.columns]
        prices['Date'] = to_datetime(
            to_datetime(prices['Date'], unit='s').dt.date)

        if 'Data' in prices.columns:
            prices = prices[prices['Data'].isnull()]
        prices = prices[['Date', 'High', 'Low', 'Open', 'Close', 'Volume',
                         'Adjclose']]
        prices = prices.rename(columns={'Adjclose': 'Adj Close'})

        prices = prices.set_index('Date')
        prices = prices.sort_index().dropna(how='all')

        if self.ret_index:
            prices['Ret_Index'] = \
                _calc_return_index(prices['Adj Close'])
        if self.adjust_price:
            prices = _adjust_prices(prices)

        # dividends & splits data
        if self.get_actions and data['eventsData']:

            actions = DataFrame(data['eventsData'])
            actions.columns = [col.capitalize() for col in actions.columns]
            actions['Date'] = to_datetime(
                to_datetime(actions['Date'], unit='s').dt.date)

            types = actions['Type'].unique()
            if 'DIVIDEND' in types:
                divs = actions[actions.Type == 'DIVIDEND'].copy()
                divs = divs[['Date', 'Amount']].reset_index(drop=True)
                divs = divs.set_index('Date')
                divs = divs.rename(columns={'Amount': 'Dividends'})
                prices = prices.join(divs, how='outer')

            if 'SPLIT' in types:
                splits = actions[actions.Type == 'SPLIT'].copy()
                splits['SplitRatio'] = splits['Splitratio'].apply(
                    lambda x: eval(x))
                splits = splits.reset_index(drop=True)
                splits = splits.set_index('Date')
                splits['Splits'] = 1.0 / splits['SplitRatio']
                prices = prices.join(splits['Splits'], how='outer')

                if 'DIVIDEND' in types and not self.adjust_dividends:
                    # Adjust dividends to deal with splits
                    adj = prices['Splits'].sort_index(ascending=False).fillna(
                        1).cumprod()
                    adj = 1.0 / adj
                    prices['Dividends'] = prices['Dividends'] * adj

        return prices


def _adjust_prices(hist_data, price_list=None):
    """
    Return modifed DataFrame with adjusted prices based on
    'Adj Close' price. Adds 'Adj_Ratio' column.
    """
    if price_list is None:
        price_list = 'Open', 'High', 'Low', 'Close'
    adj_ratio = hist_data['Adj Close'] / hist_data['Close']

    data = hist_data.copy()
    for item in price_list:
        data[item] = hist_data[item] * adj_ratio
    data['Adj_Ratio'] = adj_ratio
    del data['Adj Close']
    return data


def _calc_return_index(price_df):
    """
    Return a returns index from a input price df or series. Initial value
    (typically NaN) is set to 1.
    """
    df = price_df.pct_change().add(1).cumprod()
    mask = notnull(df.iloc[1]) & isnull(df.iloc[0])
    if mask:
        df.loc[df.index[0]] = 1

    # Check for first stock listings after starting date of index in ret_index
    # If True, find first_valid_index and set previous entry to 1.
    if not mask:
        tstamp = df.first_valid_index()
        t_idx = df.index.get_loc(tstamp) - 1
        df.iloc[t_idx] = 1

    return df



import json
from collections import OrderedDict

import pandas.compat as compat
from pandas import DataFrame


from pandas_datareader.base import _BaseReader


_DEFAULT_PARAMS = {
    'lang': 'en-US',
    'corsDomain': 'finance.yahoo.com',
    '.tsrc': 'finance',
}


class YahooQuotesReader(_BaseReader):

    """Get current yahoo quote"""

    @property
    def url(self):
        return 'https://query1.finance.yahoo.com/v7/finance/quote'

    def read(self):
        if isinstance(self.symbols, compat.string_types):
            return self._read_one_data(self.url, self.params(self.symbols))
        else:
            data = OrderedDict()
            for symbol in self.symbols:
                data[symbol] = \
                    self._read_one_data(
                        self.url, self.params(symbol)).loc[symbol]
            return DataFrame.from_dict(data, orient='index')

    def params(self, symbol):
        """Parameters to use in API calls"""
        # Construct the code request string.
        params = {'symbols': symbol}
        params.update(_DEFAULT_PARAMS)
        return params

    def _read_lines(self, out):
        data = json.loads(out.read())['quoteResponse']['result'][0]
        idx = data.pop('symbol')
        data['price'] = data['regularMarketPrice']
        return DataFrame(data, index=[idx])



from pandas_datareader.iex import IEX

# Data provided for free by IEX
# Data is furnished in compliance with the guidelines promulgated in the IEX
# API terms of service and manual
# See https://iextrading.com/api-exhibit-a/ for additional information
# and conditions of use


class TopsReader(IEX):
    """
    Near-real time aggregated bid and offer positions

    Notes
    -----
    IEX's aggregated best quoted bid and offer position for all securities
    on IEX's displayed limit order book.
    """

    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None):
        super(TopsReader, self).__init__(symbols=symbols,
                                         start=start, end=end,
                                         retry_count=retry_count,
                                         pause=pause, session=session)

    @property
    def service(self):
        """Service endpoint"""
        return "tops"


class LastReader(IEX):
    """
    Information of executions on IEX

    Notes
    -----
    Last provides trade data for executions on IEX. Provides last sale price,
    size and time.
    """
    # todo: Eventually we'll want to implement WebSockets as an option.
    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None):
        super(LastReader, self).__init__(symbols=symbols,
                                         start=start, end=end,
                                         retry_count=retry_count,
                                         pause=pause, session=session)

    @property
    def service(self):
        """Service endpoint"""
        return "tops/last"



import os
import re

from pandas_datareader.base import _DailyBaseReader


class QuandlReader(_DailyBaseReader):
    """
    Returns DataFrame of historical stock prices from symbol, over date
    range, start to end.

    .. versionadded:: 0.5.0

    Parameters
    ----------
    symbols : string
        Possible formats:
        1. DB/SYM: The Quandl 'codes': DB is the database name,
        SYM is a ticker-symbol-like Quandl abbreviation
        for a particular security.
        2. SYM.CC: SYM is the same symbol and CC is an ISO country code,
        will try to map to the best single Quandl database for that country.
        Beware of ambiguous symbols (different securities per country)!
        Note: Cannot use more than a single string because of the inflexible
        way the URL is composed of url and _get_params in the superclass
    start : string
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : string, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        Number of symbols to download consecutively before intiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used
    api_key : str, optional
        Quandl API key . If not provided the environmental variable
        QUANDL_API_KEY is read. The API key is *required*.
    """

    _BASE_URL = "https://www.quandl.com/api/v3/datasets/"

    def __init__(self, symbols, start=None, end=None, retry_count=3, pause=0.1,
                 session=None, chunksize=25, api_key=None):
        super(QuandlReader, self).__init__(symbols, start, end, retry_count,
                                           pause, session, chunksize)
        if api_key is None:
            api_key = os.getenv('QUANDL_API_KEY')
        if not api_key or not isinstance(api_key, str):
            raise ValueError('The Quandl API key must be provided either '
                             'through the api_key variable or through the '
                             'environmental variable QUANDL_API_KEY.')
        self.api_key = api_key

    @property
    def url(self):
        """API URL"""
        symbol = self.symbols if isinstance(self.symbols, str) else \
            self.symbols[0]
        mm = self._fullmatch(r"([A-Z0-9]+)(([/\.])([A-Z0-9_]+))?", symbol)
        assert mm, ("Symbol '%s' must conform to Quandl convention 'DB/SYM'" %
                    symbol)
        datasetname = 'WIKI'
        if not mm.group(2):
            # bare symbol:
            datasetname = 'WIKI'  # default; symbol stays itself
        elif mm.group(3) == "/":
            # --- normal Quandl DB/SYM convention:
            symbol = mm.group(4)
            datasetname = mm.group(1)
        elif mm.group(3) == ".":
            # secondary convention SYM.CountryCode:
            symbol = mm.group(1)
            datasetname = self._db_from_countrycode(mm.group(4))
        params = {
            'start_date': self.start.strftime('%Y-%m-%d'),
            'end_date': self.end.strftime('%Y-%m-%d'),
            'order': "asc",
            'api_key': self.api_key
        }
        paramstring = '&'.join(['%s=%s' % (k, v) for k, v in params.items()])
        url = '{url}{dataset}/{symbol}.csv?{params}'
        return url.format(url=self._BASE_URL, dataset=datasetname,
                          symbol=symbol, params=paramstring)

    def _fullmatch(self, regex, string, flags=0):
        """Emulate python-3.4 re.fullmatch()."""
        return re.match("(?:" + regex + r")\Z", string, flags=flags)

    _COUNTRYCODE_TO_DATASET = dict(
        # https://www.quandl.com/data/EURONEXT-Euronext-Stock-Exchange
        BE='EURONEXT',
        # https://www.quandl.com/data/HKEX-Hong-Kong-Exchange
        CN='HKEX',
        # https://www.quandl.com/data/SSE-Boerse-Stuttgart
        DE='SSE',
        FR='EURONEXT',
        # https://www.quandl.com/data/NSE-National-Stock-Exchange-of-India
        IN='NSE',
        # https://www.quandl.com/data/TSE-Tokyo-Stock-Exchange
        JP='TSE',
        NL='EURONEXT',
        PT='EURONEXT',
        # https://www.quandl.com/data/LSE-London-Stock-Exchange
        UK='LSE',
        # https://www.quandl.com/data/WIKI-Wiki-EOD-Stock-Prices
        US='WIKI',
    )

    def _db_from_countrycode(self, code):
        assert code in self._COUNTRYCODE_TO_DATASET, \
            "No Quandl dataset known for country code '%s'" % code
        return self._COUNTRYCODE_TO_DATASET[code]

    def _get_params(self, symbol):
        return {}

    def read(self):
        """Read data"""
        df = super(QuandlReader, self).read()
        df.rename(columns=lambda n: n.replace(' ', '')
                  .replace('.', '')
                  .replace('/', '')
                  .replace('%', '')
                  .replace('(', '')
                  .replace(')', '')
                  .replace("'", '')
                  .replace('-', ''),
                  inplace=True)
        return df



print("import: 'pandas_datareader'")
import pandas_datareader

print("import: 'pandas_datareader.compat'")
import pandas_datareader.compat

print("import: 'pandas_datareader.google'")
import pandas_datareader.google

print("import: 'pandas_datareader.iex'")
import pandas_datareader.iex

print("import: 'pandas_datareader.io'")
import pandas_datareader.io

print("import: 'pandas_datareader.mstar'")
import pandas_datareader.mstar

print("import: 'pandas_datareader.tests'")
import pandas_datareader.tests

print("import: 'pandas_datareader.tests.google'")
import pandas_datareader.tests.google

print("import: 'pandas_datareader.tests.io'")
import pandas_datareader.tests.io

print("import: 'pandas_datareader.tests.mstar'")
import pandas_datareader.tests.mstar

print("import: 'pandas_datareader.tests.yahoo'")
import pandas_datareader.tests.yahoo

print("import: 'pandas_datareader.yahoo'")
import pandas_datareader.yahoo




from datetime import datetime

import numpy as np
import pandas as pd
import pytest
import requests

import pandas_datareader.data as web
from pandas_datareader.data import MorningstarDailyReader
from pandas_datareader._utils import SymbolWarning


@pytest.mark.xfail(reason="Deprecated")
class TestMorningstarDaily(object):

    def test_invalid_date(self):
        with pytest.raises(ValueError):
            web.DataReader("MSFT", 'morningstar', start="1990-03-A")
        with pytest.raises(ValueError):
            web.DataReader("MSFT", 'morningstar', start="2001-02-02",
                           end="1999-03-03")

    def test_invalid_partial_multi_symbols(self):
        with pytest.warns(SymbolWarning):
            df = web.DataReader(['MSFT', "21##", ""],
                                "morningstar", retry_count=0)
            assert (len(df.index.levels[0]) == 1)

    def test_invalid_multi_symbols(self):
        with pytest.raises(ValueError):
            web.DataReader(["#$@", "21122"], "morningstar", retry_count=0)

    def test_invalid_symbol_type(self):
        with pytest.raises(TypeError):
            web.DataReader([12332], data_source='morningstar', retry_count=0)

    def test_mstar(self):
        start = datetime(2014, 3, 5)
        end = datetime(2018, 1, 18)
        df = web.DataReader('MSFT', 'morningstar', start=start, end=end)
        assert (df['Open'][-1] == 89.8)

    def test_get_data_single_symbol(self):
        # single symbol
        # just test that we succeed
        web.get_data_morningstar('GOOG')

    def test_get_data_interval(self):
        # daily interval data
        pan = web.get_data_morningstar(symbols='XOM', start='2013-01-01',
                                       end='2013-12-31', interval='d')
        assert len(pan) == 261

        # weekly interval data
        pan = web.get_data_morningstar(symbols='XOM', start='2013-01-01',
                                       end='2013-12-31', interval='w')
        assert len(pan) == 54

        # monthly interval data
        pan = web.get_data_morningstar(symbols='XOM', start='2013-01-01',
                                       end='2013-12-31', interval='m')
        assert len(pan) == 13

        # test fail on invalid interval
        with pytest.raises(ValueError):
            web.get_data_morningstar('XOM', interval='NOT VALID')

    def test_get_data_multiple_symbols(self):
        # just test that we succeed
        sl = ['AAPL', 'AMZN', 'GOOG']
        web.get_data_morningstar(sl, '2012')

    def test_get_data_multiple_symbols_two_dates(self):
        df = web.get_data_morningstar(symbols=['XOM', 'MSFT'],
                                      start='2013-01-01',
                                      end='2013-03-04')

        assert len(df.index.levels[0]) == 2
        assert 'XOM' in df.index.levels[0]
        assert 'MSFT' in df.index.levels[0]

        # sanity checking
        assert df.dtypes['Close'] == np.float64
        assert df.dtypes['Open'] == np.float64
        assert df.dtypes['Low'] == np.float64
        assert df.dtypes['High'] == np.float64
        assert df.dtypes['Volume'] == np.int64

    def incl_dividend_column_multi(self):
        df = web.get_data_morningstar(symbols=['XOM', 'MSFT'],
                                      start='2013-01-01',
                                      end='2013-03-04', incl_dividends=True)

        assert ("isDividend" in df)

    def incl_splits_column_multi(self):
        df = web.get_data_morningstar(symbols=['XOM', 'MSFT'],
                                      start='2013-01-01',
                                      end='2013-03-04', incl_dividends=True)
        assert ("isSplit" in df)

    def excl_volume_column_multi(self):
        df = web.get_data_morningstar(symbols=["XOM", "MSFT"],
                                      start='2013-01-01',
                                      end='2013-03-04', incl_volume=False)
        assert ("Volume" not in df.keys())

    def test_mstar_reader_class(self):
        dr = MorningstarDailyReader(symbols="GOOG", interval="d")
        df = dr.read()

        assert df.Close['GOOG']['2017-12-13'] == 1040.61

        session = requests.Session()

        dr = MorningstarDailyReader('GOOG', session=session)
        dr.read()
        assert dr.session is session

    def test_mstar_DataReader_multi(self):
        start = datetime(2010, 1, 1)
        end = datetime(2015, 5, 9)

        result = web.DataReader(['AAPL', 'F'], 'morningstar', start, end)
        assert isinstance(result, pd.DataFrame)



import datetime
import json

import pandas as pd

from dateutil.relativedelta import relativedelta
from pandas_datareader.base import _DailyBaseReader

# Data provided for free by IEX
# Data is furnished in compliance with the guidelines promulgated in the IEX
# API terms of service and manual
# See https://iextrading.com/api-exhibit-a/ for additional information
# and conditions of use


class IEXDailyReader(_DailyBaseReader):

    """
    Returns DataFrame of historical stock prices
    from symbols, over date range, start to end. To avoid being penalized by
    IEX servers, pauses between downloading 'chunks' of symbols can be
    specified.

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    start : string, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : string, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        Number of symbols to download consecutively before intiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used
    """

    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None, chunksize=25):
        super(IEXDailyReader, self).__init__(symbols=symbols, start=start,
                                             end=end, retry_count=retry_count,
                                             pause=pause, session=session,
                                             chunksize=chunksize)

    @property
    def url(self):
        """API URL"""
        return 'https://api.iextrading.com/1.0/stock/market/batch'

    @property
    def endpoint(self):
        """API endpoint"""
        return "chart"

    def _get_params(self, symbol):
        chart_range = self._range_string_from_date()
        if isinstance(symbol, list):
            symbolList = ','.join(symbol)
        else:
            symbolList = symbol
        params = {
            "symbols": symbolList,
            "types": self.endpoint,
            "range": chart_range,
        }
        return params

    def _range_string_from_date(self):
        delta = relativedelta(self.start, datetime.datetime.now())
        if 2 <= (delta.years * -1) <= 5:
            return "5y"
        elif 1 <= (delta.years * -1) <= 2:
            return "2y"
        elif 0 <= (delta.years * -1) < 1:
            return "1y"
        else:
            raise ValueError(
                "Invalid date specified. Must be within past 5 years.")

    def read(self):
        """Read data"""
        try:
            return self._read_one_data(self.url,
                                       self._get_params(self.symbols))
        finally:
            self.close()

    def _read_lines(self, out):
        data = out.read()
        json_data = json.loads(data)
        result = {}
        if type(self.symbols) is str:
            syms = [self.symbols]
        else:
            syms = self.symbols
        for symbol in syms:
            d = json_data.pop(symbol)["chart"]
            df = pd.DataFrame(d)
            df.set_index("date", inplace=True)
            values = ["open", "high", "low", "close", "volume"]
            df = df[values]
            sstart = self.start.strftime('%Y-%m-%d')
            send = self.end.strftime('%Y-%m-%d')
            df = df.loc[sstart:send]
            result.update({symbol: df})
        if len(result) > 1:
            result = pd.concat(result).unstack(level=0)
            result.columns.names = ['Attributes', 'Symbols']
            return result
        return result[self.symbols]



# pylint: disable-msg=E1101,W0613,W0603

import os

import numpy as np
import pandas as pd
import pandas.util.testing as tm
import pytest

from pandas_datareader.io.sdmx import read_sdmx, _read_sdmx_dsd


@pytest.fixture
def dirpath(datapath):
    return datapath("io", "data")


def test_tourism(dirpath):
    # Eurostat
    # Employed doctorate holders in non managerial and non professional
    # occupations by fields of science (%)
    dsd = _read_sdmx_dsd(os.path.join(dirpath, 'sdmx',
                                      'DSD_cdh_e_fos.xml'))
    df = read_sdmx(os.path.join(dirpath, 'sdmx',
                                'cdh_e_fos.xml'), dsd=dsd)

    assert isinstance(df, pd.DataFrame)
    assert df.shape == (2, 336)

    df = df['Percentage']['Total']['Natural sciences']
    df = df[['Norway', 'Poland', 'Portugal', 'Russia']]

    exp_col = pd.MultiIndex.from_product([['Norway', 'Poland', 'Portugal',
                                           'Russia'], ['Annual']],
                                         names=['GEO', 'FREQ'])
    exp_idx = pd.DatetimeIndex(['2009', '2006'], name='TIME_PERIOD')

    values = np.array([[20.38, 25.1, 27.77, 38.1],
                       [25.49, np.nan, 39.05, np.nan]])
    expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
    tm.assert_frame_equal(df, expected)



import os
import time

from pandas.compat import StringIO
import pandas.compat as compat
import pandas as pd

from pandas_datareader.base import _BaseReader


class EnigmaReader(_BaseReader):
    """
    Collects current snapshot of Enigma data located at the specified
    data set ID and returns a pandas DataFrame.

    Parameters
    ----------
    dataset_id : str
        Enigma dataset UUID.
    api_key : str, optional
        Enigma API key. If not provided, the environmental variable
        ENIGMA_API_KEY is read.
    retry_count : int, default 5
        Number of times to retry query request.
    pause : float, default 0.1
        Time, in seconds, of the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used.
    base_url : str, optional (defaults to https://public.enigma.com/api)
        Alternative Enigma endpoint to be used.

    Examples
    --------
    Download current snapshot for the following Florida Inspections Dataset:
    https://public.enigma.com/datasets/bedaf052-5fcd-4758-8d27-048ce8746c6a

    >>> import pandas_datareader as pdr
    >>> df = pdr.get_data_enigma('bedaf052-5fcd-4758-8d27-048ce8746c6a')

    In the event that ENIGMA_API_KEY does not exist in your env, the key can
    be supplied as the second argument or as the keyword argument `api_key`

    >>> df = EnigmaReader(dataset_id='bedaf052-5fcd-4758-8d27-048ce8746c6a',
    ...                   api_key='INSERT_API_KEY').read()
    """
    def __init__(self,
                 dataset_id=None,
                 api_key=None,
                 retry_count=5,
                 pause=.75,
                 session=None,
                 base_url="https://public.enigma.com/api"):

        super(EnigmaReader, self).__init__(symbols=[],
                                           retry_count=retry_count,
                                           pause=pause, session=session)
        if api_key is None:
            self._api_key = os.getenv('ENIGMA_API_KEY')
            if self._api_key is None:
                raise ValueError("Please provide an Enigma API key or set "
                                 "the ENIGMA_API_KEY environment variable\n"
                                 "If you do not have an API key, you can get "
                                 "one here: http://public.enigma.com/signup")
        else:
            self._api_key = api_key

        self._dataset_id = dataset_id
        if not isinstance(self._dataset_id, compat.string_types):
            raise ValueError(
                "The Enigma dataset_id must be a string (ex: "
                "'bedaf052-5fcd-4758-8d27-048ce8746c6a')")

        headers = {
            'Authorization': 'Bearer {0}'.format(self._api_key),
            'User-Agent': 'pandas-datareader',
        }
        self.session.headers.update(headers)
        self._base_url = base_url
        self._retry_count = retry_count
        self._retry_delay = pause

    def read(self):
        """Read data"""
        try:
            return self._read()
        finally:
            self.close()

    def _read(self):
        snapshot_id = self.get_current_snapshot_id(self._dataset_id)
        exported_data = self.get_snapshot_export(snapshot_id)  # TODO: Retry?
        decoded_data = exported_data.decode("utf-8")
        return pd.read_csv(StringIO(decoded_data))

    def _get(self, url):
        """HTTP GET Request with Retry Logic"""
        url = "{0}/{1}".format(self._base_url, url)
        attempts = 0
        while True:
            try:
                response = self.session.get(url)
                response.raise_for_status()
                return response
            except Exception as e:
                if attempts < self._retry_count:
                    attempts += 1
                    time.sleep(self._retry_delay)
                    continue
                else:
                    raise e

    def get_current_snapshot_id(self, dataset_id):
        """Get ID of the most current snapshot of a dataset"""
        dataset_metadata = self.get_dataset_metadata(dataset_id)
        return dataset_metadata['current_snapshot']['id']

    def get_dataset_metadata(self, dataset_id):
        """Get the Dataset Model of this EnigmaReader's dataset
            https://docs.public.enigma.com/resources/dataset/index.html
        """
        url = "datasets/{0}?row_limit=0".format(dataset_id)
        response = self._get(url)
        return response.json()

    def get_snapshot_export(self, snapshot_id):
        """Return raw CSV of a dataset"""
        url = "export/{0}".format(snapshot_id)
        response = self._get(url)
        return response.content



import json

import pandas as pd
from pandas.io.common import urlencode
from pandas_datareader.base import _BaseReader


# Data provided for free by IEX
# Data is furnished in compliance with the guidelines promulgated in the IEX
# API terms of service and manual
# See https://iextrading.com/api-exhibit-a/ for additional information
# and conditions of use


class IEX(_BaseReader):
    """
    Serves as the base class for all IEX API services.
    """

    _format = 'json'

    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None):
        super(IEX, self).__init__(symbols=symbols,
                                  start=start, end=end,
                                  retry_count=retry_count,
                                  pause=pause, session=session)

    @property
    def service(self):
        """Service endpoint"""
        # This property will be overridden by the subclass
        raise NotImplementedError("IEX API service not specified.")

    @property
    def url(self):
        """API URL"""
        qstring = urlencode(self._get_params(self.symbols))
        return "https://api.iextrading.com/1.0/{}?{}".format(self.service,
                                                             qstring)

    def read(self):
        """Read data"""
        df = super(IEX, self).read()
        if isinstance(df, pd.DataFrame):
            df = df.squeeze()
            if not isinstance(df, pd.DataFrame):
                df = pd.DataFrame(df)
        return df

    def _get_params(self, symbols):
        p = {}
        if isinstance(symbols, list):
            p['symbols'] = ','.join(symbols)
        elif isinstance(symbols, str):
            p['symbols'] = symbols
        return p

    def _output_error(self, out):
        """If IEX returns a non-200 status code, we need to notify the user of
        the error returned.

        :param out: Raw HTTP Output
        """
        try:
            content = json.loads(out.text)
        except Exception:
            raise TypeError("Failed to interpret response as JSON.")

        for key, string in content.items():
            e = "IEX Output error encountered: {}".format(string)
            if key == 'error':
                raise Exception(e)

    def _read_lines(self, out):
        """IEX's output does not need anything complex, so we're overriding to
        use Pandas' default interpreter

        :param out: Raw HTTP Output
        :return: DataFrame
        """

        # IEX will return a blank line for invalid tickers:
        if isinstance(out, list):
            out = [x for x in out if x is not None]
        return pd.DataFrame(out) if len(out) > 0 else pd.DataFrame()



from pandas_datareader.base import _DailyBaseReader


class StooqDailyReader(_DailyBaseReader):

    """
    Returns DataFrame/dict of Dataframes of historical stock prices from
    symbols, over date range, start to end.

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    start: string, date which to start interval at YYYYMMDD.
    end: string, date which to end interval at YYYYMMDD.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        Number of symbols to download consecutively before initiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used
    freq: string, d, w, m ,q, y for daily, weekly, monthly, quarterly, yearly

    Notes
    -----
    See `Stooq <https://stooq.com>`__
    """

    @property
    def url(self):
        """API URL"""
        return 'https://stooq.com/q/d/l/'

    def _get_params(self, symbol, country='US'):
        symbol_parts = symbol.split(".")
        if len(symbol_parts) == 1:
            symbol = ".".join([symbol, country])
        else:
            if symbol_parts[1].lower() not in ['de', 'hk', 'hu', 'jp',
                                               'pl', 'uk', 'us']:
                symbol = ".".join([symbol, 'US'])

        params = {
            's': symbol,
            'i': self.freq or 'd',
            'd1': self.start.strftime('%Y%m%d'),
            'd2': self.end.strftime('%Y%m%d')
        }

        return params



# -*- coding: utf-8 -*-

import warnings

from pandas.compat import reduce, lrange, string_types
import pandas as pd
import numpy as np

from pandas_datareader.base import _BaseReader

# This list of country codes was pulled from wikipedia during October 2014.
# While some exceptions do exist, it is the best proxy for countries supported
# by World Bank.  It is an aggregation of the 2-digit ISO 3166-1 alpha-2, and
# 3-digit ISO 3166-1 alpha-3, codes, with 'all', 'ALL', and 'All' appended ot
# the end.

WB_API_URL = 'https://api.worldbank.org/v2'

country_codes = ['AD', 'AE', 'AF', 'AG', 'AI', 'AL', 'AM', 'AO', 'AQ', 'AR',
                 'AS', 'AT', 'AU', 'AW', 'AX', 'AZ', 'BA', 'BB', 'BD', 'BE',
                 'BF', 'BG', 'BH', 'BI', 'BJ', 'BL', 'BM', 'BN', 'BO', 'BQ',
                 'BR', 'BS', 'BT', 'BV', 'BW', 'BY', 'BZ', 'CA', 'CC', 'CD',
                 'CF', 'CG', 'CH', 'CI', 'CK', 'CL', 'CM', 'CN', 'CO', 'CR',
                 'CU', 'CV', 'CW', 'CX', 'CY', 'CZ', 'DE', 'DJ', 'DK', 'DM',
                 'DO', 'DZ', 'EC', 'EE', 'EG', 'EH', 'ER', 'ES', 'ET', 'FI',
                 'FJ', 'FK', 'FM', 'FO', 'FR', 'GA', 'GB', 'GD', 'GE', 'GF',
                 'GG', 'GH', 'GI', 'GL', 'GM', 'GN', 'GP', 'GQ', 'GR', 'GS',
                 'GT', 'GU', 'GW', 'GY', 'HK', 'HM', 'HN', 'HR', 'HT', 'HU',
                 'ID', 'IE', 'IL', 'IM', 'IN', 'IO', 'IQ', 'IR', 'IS', 'IT',
                 'JE', 'JM', 'JO', 'JP', 'KE', 'KG', 'KH', 'KI', 'KM', 'KN',
                 'KP', 'KR', 'KW', 'KY', 'KZ', 'LA', 'LB', 'LC', 'LI', 'LK',
                 'LR', 'LS', 'LT', 'LU', 'LV', 'LY', 'MA', 'MC', 'MD', 'ME',
                 'MF', 'MG', 'MH', 'MK', 'ML', 'MM', 'MN', 'MO', 'MP', 'MQ',
                 'MR', 'MS', 'MT', 'MU', 'MV', 'MW', 'MX', 'MY', 'MZ', 'NA',
                 'NC', 'NE', 'NF', 'NG', 'NI', 'NL', 'NO', 'NP', 'NR', 'NU',
                 'NZ', 'OM', 'PA', 'PE', 'PF', 'PG', 'PH', 'PK', 'PL', 'PM',
                 'PN', 'PR', 'PS', 'PT', 'PW', 'PY', 'QA', 'RE', 'RO', 'RS',
                 'RU', 'RW', 'SA', 'SB', 'SC', 'SD', 'SE', 'SG', 'SH', 'SI',
                 'SJ', 'SK', 'SL', 'SM', 'SN', 'SO', 'SR', 'SS', 'ST', 'SV',
                 'SX', 'SY', 'SZ', 'TC', 'TD', 'TF', 'TG', 'TH', 'TJ', 'TK',
                 'TL', 'TM', 'TN', 'TO', 'TR', 'TT', 'TV', 'TW', 'TZ', 'UA',
                 'UG', 'UM', 'US', 'UY', 'UZ', 'VA', 'VC', 'VE', 'VG', 'VI',
                 'VN', 'VU', 'WF', 'WS', 'YE', 'YT', 'ZA', 'ZM', 'ZW',
                 'ABW', 'AFG', 'AGO', 'AIA', 'ALA', 'ALB', 'AND', 'ARE',
                 'ARG', 'ARM', 'ASM', 'ATA', 'ATF', 'ATG', 'AUS', 'AUT',
                 'AZE', 'BDI', 'BEL', 'BEN', 'BES', 'BFA', 'BGD', 'BGR',
                 'BHR', 'BHS', 'BIH', 'BLM', 'BLR', 'BLZ', 'BMU', 'BOL',
                 'BRA', 'BRB', 'BRN', 'BTN', 'BVT', 'BWA', 'CAF', 'CAN',
                 'CCK', 'CHE', 'CHL', 'CHN', 'CIV', 'CMR', 'COD', 'COG',
                 'COK', 'COL', 'COM', 'CPV', 'CRI', 'CUB', 'CUW', 'CXR',
                 'CYM', 'CYP', 'CZE', 'DEU', 'DJI', 'DMA', 'DNK', 'DOM',
                 'DZA', 'ECU', 'EGY', 'ERI', 'ESH', 'ESP', 'EST', 'ETH',
                 'FIN', 'FJI', 'FLK', 'FRA', 'FRO', 'FSM', 'GAB', 'GBR',
                 'GEO', 'GGY', 'GHA', 'GIB', 'GIN', 'GLP', 'GMB', 'GNB',
                 'GNQ', 'GRC', 'GRD', 'GRL', 'GTM', 'GUF', 'GUM', 'GUY',
                 'HKG', 'HMD', 'HND', 'HRV', 'HTI', 'HUN', 'IDN', 'IMN',
                 'IND', 'IOT', 'IRL', 'IRN', 'IRQ', 'ISL', 'ISR', 'ITA',
                 'JAM', 'JEY', 'JOR', 'JPN', 'KAZ', 'KEN', 'KGZ', 'KHM',
                 'KIR', 'KNA', 'KOR', 'KWT', 'LAO', 'LBN', 'LBR', 'LBY',
                 'LCA', 'LIE', 'LKA', 'LSO', 'LTU', 'LUX', 'LVA', 'MAC',
                 'MAF', 'MAR', 'MCO', 'MDA', 'MDG', 'MDV', 'MEX', 'MHL',
                 'MKD', 'MLI', 'MLT', 'MMR', 'MNE', 'MNG', 'MNP', 'MOZ',
                 'MRT', 'MSR', 'MTQ', 'MUS', 'MWI', 'MYS', 'MYT', 'NAM',
                 'NCL', 'NER', 'NFK', 'NGA', 'NIC', 'NIU', 'NLD', 'NOR',
                 'NPL', 'NRU', 'NZL', 'OMN', 'PAK', 'PAN', 'PCN', 'PER',
                 'PHL', 'PLW', 'PNG', 'POL', 'PRI', 'PRK', 'PRT', 'PRY',
                 'PSE', 'PYF', 'QAT', 'REU', 'ROU', 'RUS', 'RWA', 'SAU',
                 'SDN', 'SEN', 'SGP', 'SGS', 'SHN', 'SJM', 'SLB', 'SLE',
                 'SLV', 'SMR', 'SOM', 'SPM', 'SRB', 'SSD', 'STP', 'SUR',
                 'SVK', 'SVN', 'SWE', 'SWZ', 'SXM', 'SYC', 'SYR', 'TCA',
                 'TCD', 'TGO', 'THA', 'TJK', 'TKL', 'TKM', 'TLS', 'TON',
                 'TTO', 'TUN', 'TUR', 'TUV', 'TWN', 'TZA', 'UGA', 'UKR',
                 'UMI', 'URY', 'USA', 'UZB', 'VAT', 'VCT', 'VEN', 'VGB',
                 'VIR', 'VNM', 'VUT', 'WLF', 'WSM', 'YEM', 'ZAF', 'ZMB',
                 'ZWE', 'all', 'ALL', 'All']


class WorldBankReader(_BaseReader):

    """
    Download data series from the World Bank's World Development Indicators

    Parameters
    ----------
    symbols: WorldBank indicator string or list of strings
        taken from the ``id`` field in ``WDIsearch()``
    countries: string or list of strings.
        ``all`` downloads data for all countries
        2 or 3 character ISO country codes select individual
        countries (e.g.``US``,``CA``) or (e.g.``USA``,``CAN``).  The codes
        can be mixed.
        The two ISO lists of countries, provided by wikipedia, are hardcoded
        into pandas as of 11/10/2014.
    start: Timestamp or int
        First year of the data series. Month and day are ignored.
    end: Timestamp or int
        Last year of the data series (inclusive). Month and day are ignored.
    errors: str {'ignore', 'warn', 'raise'}, default 'warn'
        Country codes are validated against a hardcoded list.  This controls
        the outcome of that validation, and attempts to also apply
        to the results from world bank.
        errors='raise', will raise a ValueError on a bad country code.
    """

    _format = 'json'

    def __init__(self, symbols=None, countries=None,
                 start=None, end=None, freq=None,
                 retry_count=3, pause=0.1, session=None, errors='warn'):

        if symbols is None:
            symbols = ['NY.GDP.MKTP.CD', 'NY.GNS.ICTR.ZS']
        elif isinstance(symbols, string_types):
            symbols = [symbols]

        super(WorldBankReader, self).__init__(symbols=symbols,
                                              start=start, end=end,
                                              retry_count=retry_count,
                                              pause=pause, session=session)

        if countries is None:
            countries = ['MX', 'CA', 'US']
        elif isinstance(countries, string_types):
            countries = [countries]

        bad_countries = np.setdiff1d(countries, country_codes)
        # Validate the input
        if len(bad_countries) > 0:
            tmp = ", ".join(bad_countries)
            if errors == 'raise':
                raise ValueError("Invalid Country Code(s): %s" % tmp)
            if errors == 'warn':
                warnings.warn('Non-standard ISO '
                              'country codes: %s' % tmp, UserWarning)

        freq_symbols = ['M', 'Q', 'A', None]

        if freq not in freq_symbols:
            msg = 'The frequency `{0}` is not in the accepted ' \
                  'list.'.format(freq)
            raise ValueError(msg)

        self.freq = freq
        self.countries = countries
        self.errors = errors

    @property
    def url(self):
        """API URL"""
        countries = ';'.join(self.countries)
        return WB_API_URL + '/countries/' + countries + '/indicators/'

    @property
    def params(self):
        """Parameters to use in API calls"""
        if self.freq == 'M':
            return {'date': '{0}M{1:02d}:{2}M{3:02d}'.format(self.start.year,
                    self.start.month, self.end.year, self.end.month),
                    'per_page': 25000, 'format': 'json'}
        elif self.freq == 'Q':
            return {'date': '{0}Q{1}:{2}Q{3}'.format(self.start.year,
                    self.start.quarter, self.end.year,
                    self.end.quarter), 'per_page': 25000,
                    'format': 'json'}
        else:
            return {'date': '{0}:{1}'.format(self.start.year, self.end.year),
                    'per_page': 25000, 'format': 'json'}

    def read(self):
        """Read data"""
        try:
            return self._read()
        finally:
            self.close()

    def _read(self):
        data = []
        for indicator in self.symbols:
            # Build URL for api call
            try:
                df = self._read_one_data(self.url + indicator, self.params)
                df.columns = ['country', 'iso_code', 'year', indicator]
                data.append(df)

            except ValueError as e:
                msg = str(e) + ' Indicator: ' + indicator
                if self.errors == 'raise':
                    raise ValueError(msg)
                elif self.errors == 'warn':
                    warnings.warn(msg)

        # Confirm we actually got some data, and build Dataframe
        if len(data) > 0:
            out = reduce(lambda x, y: x.merge(y, how='outer'), data)
            out = out.drop('iso_code', axis=1)
            out = out.set_index(['country', 'year'])
            out = out.apply(pd.to_numeric, errors='ignore')

            return out
        else:
            msg = "No indicators returned data."
            raise ValueError(msg)

    def _read_lines(self, out):
        # Check to see if there is a possible problem
        possible_message = out[0]

        if 'message' in possible_message.keys():
            msg = possible_message['message'][0]
            try:
                msg = msg['key'].split() + ["\n "] + msg['value'].split()
                wb_err = ' '.join(msg)
            except Exception:
                wb_err = ""
                if 'key' in msg.keys():
                    wb_err = msg['key'] + "\n "
                if 'value' in msg.keys():
                    wb_err += msg['value']

            msg = "Problem with a World Bank Query \n %s." % wb_err
            raise ValueError(msg)

        if 'total' in possible_message.keys():
            if possible_message['total'] == 0:
                msg = "No results found from world bank."
                raise ValueError(msg)

        # Parse JSON file
        data = out[1]
        country = [x['country']['value'] for x in data]
        iso_code = [x['country']['id'] for x in data]
        year = [x['date'] for x in data]
        value = [x['value'] for x in data]
        # Prepare output
        df = pd.DataFrame([country, iso_code, year, value]).T
        return df

    def get_countries(self):
        """Query information about countries

        Notes
        -----
        Provides information such as:

          * country code
          * region
          * income level
          * capital city
          * latitude
          * and longitude

        """
        url = WB_API_URL + '/countries/?per_page=1000&format=json'

        resp = self._get_response(url)
        data = resp.json()[1]

        data = pd.DataFrame(data)
        data.adminregion = [x['value'] for x in data.adminregion]
        data.incomeLevel = [x['value'] for x in data.incomeLevel]
        data.lendingType = [x['value'] for x in data.lendingType]
        data.region = [x['value'] for x in data.region]
        data.latitude = [float(x) if x != "" else np.nan
                         for x in data.latitude]
        data.longitude = [float(x) if x != "" else np.nan
                          for x in data.longitude]
        data = data.rename(columns={'id': 'iso3c', 'iso2Code': 'iso2c'})
        return data

    def get_indicators(self):
        """Download information about all World Bank data series"""
        global _cached_series
        if isinstance(_cached_series, pd.DataFrame):
            return _cached_series.copy()

        url = WB_API_URL + '/indicators?per_page=50000&format=json'

        resp = self._get_response(url)
        data = resp.json()[1]

        data = pd.DataFrame(data)
        # Clean fields
        data.source = [x['value'] for x in data.source]

        def encode_ascii(x):
            return x.encode('ascii', 'ignore')

        data.sourceOrganization = data.sourceOrganization.apply(encode_ascii)
        # Clean topic field

        def get_value(x):
            try:
                return x['value']
            except Exception:
                return ''

        def get_list_of_values(x):
            return [get_value(y) for y in x]

        data.topics = data.topics.apply(get_list_of_values)
        data.topics = data.topics.apply(lambda x: ' ; '.join(x))

        # Clean output
        data = data.sort_values(by='id')
        data.index = pd.Index(lrange(data.shape[0]))

        # cache
        _cached_series = data.copy()

        return data

    def search(self, string='gdp.*capi', field='name', case=False):
        """
        Search available data series from the world bank

        Parameters
        ----------
        string: string
            regular expression
        field: string
            id, name, source, sourceNote, sourceOrganization, topics
            See notes below
        case: bool
            case sensitive search?

        Notes
        -----
        The first time this function is run it will download and cache the full
        list of available series. Depending on the speed of your network
        connection, this can take time. Subsequent searches will use the cached
        copy, so they should be much faster.

        id : Data series indicator (for use with the ``indicator`` argument of
        ``WDI()``) e.g. NY.GNS.ICTR.GN.ZS"
        name: Short description of the data series
        source: Data collection project
        sourceOrganization: Data collection organization
        note:
        sourceNote:
        topics:
        """
        indicators = self.get_indicators()
        data = indicators[field]
        idx = data.str.contains(string, case=case)
        out = indicators.loc[idx].dropna()
        return out


def download(country=None, indicator=None, start=2003, end=2005, freq=None,
             errors='warn', **kwargs):
    """
    Download data series from the World Bank's World Development Indicators

    Parameters
    ----------
    indicator: string or list of strings
        taken from the ``id`` field in ``WDIsearch()``
    country: string or list of strings.
        ``all`` downloads data for all countries
        2 or 3 character ISO country codes select individual
        countries (e.g.``US``,``CA``) or (e.g.``USA``,``CAN``).  The codes
        can be mixed.

        The two ISO lists of countries, provided by wikipedia, are hardcoded
        into pandas as of 11/10/2014.
    start: int
        First year of the data series
    end: int
        Last year of the data series (inclusive)
    freq: str
        frequency or periodicity of the data to be retrieved (e.g. 'M' for
        monthly, 'Q' for quarterly, and 'A' for annual). None defaults to
        annual.
    errors: str {'ignore', 'warn', 'raise'}, default 'warn'
        Country codes are validated against a hardcoded list.  This controls
        the outcome of that validation, and attempts to also apply
        to the results from world bank.
        errors='raise', will raise a ValueError on a bad country code.
    kwargs:
        keywords passed to WorldBankReader

    Returns
    -------
    data : DataFrame
        DataFrame with columns country, iso_code, year, indicator value
    """
    return WorldBankReader(symbols=indicator, countries=country,
                           start=start, end=end, freq=freq, errors=errors,
                           **kwargs).read()


def get_countries(**kwargs):
    """Query information about countries

    Provides information such as:
        country code, region, income level,
        capital city, latitude, and longitude

    Parameters
    ----------

    kwargs:
        keywords passed to WorldBankReader

    """
    return WorldBankReader(**kwargs).get_countries()


def get_indicators(**kwargs):
    """Download information about all World Bank data series

    Parameters
    ----------

    kwargs:
        keywords passed to WorldBankReader

    """
    return WorldBankReader(**kwargs).get_indicators()


_cached_series = None


def search(string='gdp.*capi', field='name', case=False, **kwargs):
    """
    Search available data series from the world bank

    Parameters
    ----------
    string: string
        regular expression
    field: string
        id, name, source, sourceNote, sourceOrganization, topics. See notes
    case: bool
        case sensitive search?
    kwargs:
        keywords passed to WorldBankReader

    Notes
    -----
    The first time this function is run it will download and cache the full
    list of available series. Depending on the speed of your network
    connection, this can take time. Subsequent searches will use the cached
    copy, so they should be much faster.

    id : Data series indicator (for use with the ``indicator`` argument of
    ``WDI()``) e.g. NY.GNS.ICTR.GN.ZS"

      * name: Short description of the data series
      * source: Data collection project
      * sourceOrganization: Data collection organization
      * note:
      * sourceNote:
      * topics:
    """

    return WorldBankReader(**kwargs).search(string=string, field=field,
                                            case=case)



from ._version import get_versions
from .data import (DataReader, Options, get_components_yahoo,
                   get_dailysummary_iex, get_data_enigma, get_data_famafrench,
                   get_data_fred, get_data_google, get_data_moex,
                   get_data_morningstar, get_data_quandl, get_data_stooq,
                   get_data_yahoo, get_data_yahoo_actions, get_iex_book,
                   get_iex_symbols, get_last_iex, get_markets_iex,
                   get_nasdaq_symbols,
                   get_quote_google, get_quote_yahoo, get_recent_iex,
                   get_records_iex, get_summary_iex, get_tops_iex,
                   get_data_tiingo, get_data_alphavantage)

__version__ = get_versions()['version']
del get_versions

__all__ = ['__version__', 'get_components_yahoo', 'get_data_enigma',
           'get_data_famafrench', 'get_data_google', 'get_data_yahoo',
           'get_data_yahoo_actions', 'get_quote_google', 'get_quote_yahoo',
           'get_iex_book', 'get_iex_symbols', 'get_last_iex',
           'get_markets_iex', 'get_recent_iex', 'get_records_iex',
           'get_summary_iex', 'get_tops_iex',
           'get_nasdaq_symbols', 'get_data_quandl', 'get_data_moex',
           'get_data_fred', 'get_dailysummary_iex', 'get_data_morningstar',
           'get_data_stooq', 'DataReader', 'Options',
           'get_data_tiingo', 'get_data_alphavantage']



import os

import pytest


@pytest.fixture
def datapath(request):
    """Get the path to a data file.

    Parameters
    ----------
    path : str
        Path to the file, relative to ``pandas/tests/``

    Returns
    -------
    path : path including ``pandas/tests``.

    Raises
    ------
    ValueError
        If the path doesn't exist and the --strict-data-files option is set.
    """
    BASE_PATH = os.path.join(os.path.dirname(__file__), 'tests')

    def deco(*args):
        path = os.path.join(BASE_PATH, *args)
        if not os.path.exists(path):
            if request.config.getoption("--strict-data-files"):
                msg = "Could not find file {} and --strict-data-files is set."
                raise ValueError(msg.format(path))
            else:
                msg = "Could not find {}."
                pytest.skip(msg.format(path))
        return path
    return deco



import pandas as pd

from pandas_datareader.av import AlphaVantage
from pandas_datareader._utils import RemoteDataError


class AVSectorPerformanceReader(AlphaVantage):
    """
    Returns DataFrame of the Alpha Vantage Sector Performances SECTOR data.

    .. versionadded:: 0.7.0

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series)
        Single currency pair (formatted 'FROM/TO') or list of the same.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    api_key : str, optional
        Alpha Vantage API key . If not provided the environmental variable
        ALPHAVANTAGE_API_KEY is read. The API key is *required*.
    """
    @property
    def function(self):
        return 'SECTOR'

    def _read_lines(self, out):
        if "Information" in out:
            raise RemoteDataError()
        else:
            out.pop("Meta Data")
        df = pd.DataFrame(out)
        columns = ["RT", "1D", "5D", "1M", "3M", "YTD", "1Y", "3Y", "5Y",
                   "10Y"]
        df.columns = columns
        return df



import pytest

import numpy as np
import pandas as pd
import pandas_datareader.data as web

from pandas import compat
from datetime import datetime
from pandas_datareader.data import GoogleDailyReader
from pandas_datareader._utils import RemoteDataError, SymbolWarning

import requests

import warnings
import pandas.util.testing as tm
from pandas.util.testing import assert_series_equal

pytestmark = pytest.mark.filterwarnings(
    'ignore::pandas_datareader.exceptions.UnstableAPIWarning')


@pytest.mark.xfail(reason="Deprecated")
def assert_n_failed_equals_n_null_columns(wngs, obj, cls=SymbolWarning):
    all_nan_cols_dict = {}

    for k, v in compat.iteritems(obj):
        is_null = pd.isnull(v)

        if hasattr(is_null, 'all'):
            is_null = is_null.all()

        all_nan_cols_dict[k] = is_null

    all_nan_cols = pd.Series(all_nan_cols_dict)

    n_all_nan_cols = all_nan_cols.sum()
    valid_warnings = pd.Series([wng for wng in wngs if wng.category == cls])

    assert len(valid_warnings) == n_all_nan_cols

    failed_symbols = all_nan_cols[all_nan_cols].index
    msgs = valid_warnings.map(lambda x: x.message)

    assert msgs.str.contains('|'.join(failed_symbols)).all()


@pytest.mark.xfail(reason="Deprecated")
class TestGoogle(object):

    @classmethod
    def setup_class(cls):
        cls.locales = tm.get_locales(prefix='en_US')
        if not cls.locales:  # pragma: no cover
            pytest.skip("US English locale not available for testing")

    @classmethod
    def teardown_class(cls):
        del cls.locales

    def test_google(self):

        # asserts that google is minimally working and that it throws
        # an exception when DataReader can't get a 200 response from
        # google

        start = datetime(2010, 1, 1)
        end = datetime(2013, 1, 27)
        for locale in self.locales:
            with tm.set_locale(locale):
                panel = web.DataReader("NYSE:F", 'google', start, end)
                assert panel.Close[-1] == 13.68

            with pytest.raises(Exception):
                web.DataReader('NON EXISTENT TICKER', 'google', start, end)

    def assert_option_result(self, df):
        """
        Validate returned quote data has expected format.
        """
        assert isinstance(df, pd.DataFrame)
        assert len(df) > 0

        exp_columns = pd.Index(['change_pct', 'last', 'time'])
        tm.assert_index_equal(df.columns, exp_columns)

        dtypes = [np.dtype(x) for x in ['float64', 'float64',
                                        'datetime64[ns]']]
        tm.assert_series_equal(df.dtypes, pd.Series(dtypes, index=exp_columns))

    def test_get_quote_string(self):
        df = web.get_quote_google('GOOG')
        assert df.loc['GOOG', 'last'] > 0.0
        tm.assert_index_equal(df.index, pd.Index(['GOOG']))
        self.assert_option_result(df)

    def test_get_quote_stringlist(self):
        df = web.get_quote_google(['GOOG', 'AMZN', 'GOOG'])
        assert_series_equal(df.iloc[0], df.iloc[2])
        tm.assert_index_equal(df.index, pd.Index(['GOOG', 'AMZN', 'GOOG']))
        self.assert_option_result(df)

    def test_get_goog_volume(self):

        for locale in self.locales:
            with tm.set_locale(locale):
                df = web.get_data_google('GOOG').sort_index()
            assert df.Volume.loc['JAN-02-2015'] == 1446662

    def test_get_multi1(self):
        for locale in self.locales:
            sl = ['AAPL', 'AMZN', 'GOOG']
            with tm.set_locale(locale):
                pan = web.get_data_google(sl, '2012')
            ts = pan.Close.GOOG.index[pan.Close.AAPL < pan.Close.GOOG]
            if (hasattr(pan, 'Close') and hasattr(pan.Close, 'GOOG') and
                    hasattr(pan.Close, 'AAPL')):
                assert ts[0].dayofyear == 3
            else:  # pragma: no cover
                with pytest.raises(AttributeError):
                    pan.Close()

    def test_get_multi_invalid(self):
        sl = ['AAPL', 'AMZN', 'INVALID']
        data = web.get_data_google(sl, '2012')
        assert 'INVALID' in data.columns.levels[1]

    def test_get_multi_all_invalid(self):
        sl = ['INVALID', 'INVALID2', 'INVALID3']
        with pytest.raises(RemoteDataError):
            web.get_data_google(sl, '2012')

    def test_get_multi2(self):
        with warnings.catch_warnings(record=True) as w:
            for locale in self.locales:
                with tm.set_locale(locale):
                    pan = web.get_data_google(['GE', 'MSFT', 'INTC'],
                                              'JAN-01-12', 'JAN-31-12')
                result = pan.Close.loc['01-18-12']
                assert_n_failed_equals_n_null_columns(w, result)

                # sanity checking

                assert np.issubdtype(result.dtype, np.floating)
                result = pan.Open.loc['Jan-15-12':'Jan-20-12']

                assert result.shape == (4, 3)
                assert_n_failed_equals_n_null_columns(w, result)

    def test_dtypes(self):
        # see gh-3995, gh-8980
        data = web.get_data_google(
            'NYSE:F',
            start='JAN-01-10',
            end='JAN-27-13')
        assert np.issubdtype(data.Open.dtype, np.number)
        assert np.issubdtype(data.Close.dtype, np.number)
        assert np.issubdtype(data.Low.dtype, np.number)
        assert np.issubdtype(data.High.dtype, np.number)
        assert np.issubdtype(data.Volume.dtype, np.number)

    def test_unicode_date(self):
        # see gh-8967

        data = web.get_data_google(
            'NYSE:F',
            start='JAN-01-10',
            end='JAN-27-13')
        assert data.index.name == 'Date'

    def test_google_reader_class(self):

        r = GoogleDailyReader('GOOG')
        df = r.read()
        assert df.Volume.loc['JAN-02-2015'] == 1446662

        session = requests.Session()
        r = GoogleDailyReader('GOOG', session=session)
        assert r.session is session

    def test_bad_retry_count(self):

        with pytest.raises(ValueError):
            web.get_data_google('NYSE:F', retry_count=-1)



from datetime import datetime

import pytest
from pandas import DataFrame

from pandas_datareader.data import (DataReader, get_summary_iex, get_last_iex,
                                    get_dailysummary_iex, get_iex_symbols,
                                    get_iex_book)
from pandas_datareader.exceptions import UnstableAPIWarning


class TestIEX(object):
    @classmethod
    def setup_class(cls):
        pytest.importorskip("lxml")

    def test_read_iex(self):
        gs = DataReader("GS", "iex-last")
        assert isinstance(gs, DataFrame)

    def test_historical(self):
        df = get_summary_iex(start=datetime(2017, 4, 1),
                             end=datetime(2017, 4, 30))
        assert df.T["averageDailyVolume"].iloc[0] == 137650908.9

    def test_false_ticker(self):
        df = get_last_iex("INVALID TICKER")
        assert df.shape[0] == 0

    @pytest.mark.xfail(reason='IEX daily history API is returning 500 as of '
                              'Jan 2018')
    def test_daily(self):
        with pytest.warns(UnstableAPIWarning):
            df = get_dailysummary_iex(start=datetime(2017, 5, 5),
                                      end=datetime(2017, 5, 6))
            assert df['routedVolume'].iloc[0] == 39974788

    def test_symbols(self):
        df = get_iex_symbols()
        assert 'GS' in df.symbol.values

    def test_live_prices(self):
        dftickers = get_iex_symbols()
        tickers = dftickers[:5].symbol.values
        df = get_last_iex(tickers[:5])
        assert df["price"].mean() > 0

    def test_deep(self):
        dob = get_iex_book('GS', service='book')
        if dob:
            assert 'GS' in dob
        else:
            pytest.xfail(reason='Can only get Book when market open')



from pandas_datareader.av import AlphaVantage

import pandas as pd
import numpy as np


class AVQuotesReader(AlphaVantage):
    """
    Returns DataFrame of Alpha Vantage Realtime Stock quotes for a symbol or
    list of symbols.

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0.1
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    """
    def __init__(self, symbols=None, retry_count=3, pause=0.1, session=None,
                 api_key=None):
        if isinstance(symbols, str):
            syms = [symbols]
        elif isinstance(symbols, list):
            if len(symbols) > 100:
                raise ValueError("Up to 100 symbols at once are allowed.")
            else:
                syms = symbols
        super(AVQuotesReader, self).__init__(symbols=syms,
                                             start=None, end=None,
                                             retry_count=retry_count,
                                             pause=pause,
                                             session=session,
                                             api_key=api_key)

    @property
    def function(self):
        return 'BATCH_STOCK_QUOTES'

    @property
    def data_key(self):
        return 'Stock Quotes'

    @property
    def params(self):
        return {
            'symbols': ','.join(self.symbols),
            'function': self.function,
            'apikey': self.api_key,
        }

    def _read_lines(self, out):
        result = []
        quotes = out[self.data_key]
        for quote in quotes:
            df = pd.DataFrame(quote, index=[0])
            df.columns = [col[3:] for col in df.columns]
            df.set_index("symbol", inplace=True)
            df["price"] = df["price"].astype('float64')
            try:
                df["volume"] = df["volume"].astype('int64')
            except ValueError:
                df["volume"] = [np.nan * len(self.symbols)]
            result.append(df)
        if len(result) != len(self.symbols):
            raise ValueError("Not all symbols downloaded. Check valid "
                             "ticker(s).")
        else:
            return pd.concat(result)



import os
import pytest

import pandas as pd

from pandas_datareader import data as web

from datetime import datetime

TEST_API_KEY = os.getenv('ALPHAVANTAGE_API_KEY')
TEST_API_KEY = None if not TEST_API_KEY else TEST_API_KEY


class TestAVTimeSeries(object):

    @classmethod
    def setup_class(cls):
        pytest.importorskip('lxml')
        cls.col_index_adj = pd.Index(["open", "high", "low", "close",
                                      "adjusted close", "volume",
                                      "dividend amount"])
        cls.col_index = pd.Index(["open", "high", "low", "close", "volume"])

    @property
    def start(self):
        return datetime(2015, 2, 9)

    @property
    def end(self):
        return datetime(2017, 5, 24)

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_av_bad_symbol(self):
        with pytest.raises(ValueError):
            web.DataReader("BADTICKER", "av-daily", start=self.start,
                           end=self.end)

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_av_daily(self):
        df = web.DataReader("AAPL", "av-daily", start=self.start, end=self.end)
        assert df.columns.equals(self.col_index)
        assert len(df) == 578
        assert df["volume"][-1] == 19118319

        expected1 = df.loc["2017-02-09"]
        assert expected1["close"] == 132.42
        assert expected1["high"] == 132.445

        expected2 = df.loc["2017-05-24"]
        assert expected2["close"] == 153.34
        assert expected2["high"] == 154.17

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_av_daily_adjusted(self):
        df = web.DataReader("AAPL", "av-daily-adjusted", start=self.start,
                            end=self.end)
        assert df.columns.equals(pd.Index(["open", "high", "low", "close",
                                           "adjusted close", "volume",
                                           "dividend amount",
                                           "split coefficient"]))
        assert len(df) == 578
        assert df["volume"][-1] == 19118319

        expected1 = df.loc["2017-02-09"]
        assert expected1["close"] == 132.42
        assert expected1["high"] == 132.445
        assert expected1["adjusted close"] == 130.3505
        assert expected1["dividend amount"] == 0.57
        assert expected1["split coefficient"] == 1.0

        expected2 = df.loc["2017-05-24"]
        assert expected2["close"] == 153.34
        assert expected2["high"] == 154.17
        assert expected2["adjusted close"] == 151.5612
        assert expected2["dividend amount"] == 0.00
        assert expected2["split coefficient"] == 1.0

    @staticmethod
    def _helper_df_weekly_monthly(df, adj=False):

        expected1 = df.loc["2015-02-27"]
        assert expected1["close"] == 128.46
        assert expected1["high"] == 133.60
        if adj:
            assert expected1["adjusted close"] == 121.5859

        expected2 = df.loc["2017-03-31"]
        assert expected2["close"] == 143.66
        assert expected2["high"] == 144.5
        if adj:
            assert expected2["adjusted close"] == 141.4148
            assert expected2["dividend amount"] == 0.00

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_av_weekly(self):
        df = web.DataReader("AAPL", "av-weekly", start=self.start,
                            end=self.end)

        assert len(df) == 119
        assert df.iloc[0].name == '2015-02-13'
        assert df.iloc[-1].name == '2017-05-19'
        assert df.columns.equals(self.col_index)
        self._helper_df_weekly_monthly(df, adj=False)

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_av_weekly_adjusted(self):
        df = web.DataReader("AAPL", "av-weekly-adjusted", start=self.start,
                            end=self.end)

        assert len(df) == 119
        assert df.iloc[0].name == '2015-02-13'
        assert df.iloc[-1].name == '2017-05-19'
        assert df.columns.equals(self.col_index_adj)
        self._helper_df_weekly_monthly(df, adj=True)

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_av_monthly(self):
        df = web.DataReader("AAPL", "av-monthly", start=self.start,
                            end=self.end)

        assert len(df) == 27
        assert df.iloc[0].name == '2015-02-27'
        assert df.iloc[-1].name == '2017-04-28'
        assert df.columns.equals(self.col_index)
        self._helper_df_weekly_monthly(df, adj=False)

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_av_monthly_adjusted(self):
        df = web.DataReader("AAPL", "av-monthly-adjusted", start=self.start,
                            end=self.end)

        assert df.columns.equals(self.col_index_adj)
        assert len(df) == 27
        assert df.iloc[0].name == '2015-02-27'
        assert df.iloc[-1].name == '2017-04-28'
        self._helper_df_weekly_monthly(df, adj=True)



from datetime import datetime
import requests

import numpy as np
import pandas as pd
from pandas import DataFrame
from requests.exceptions import ConnectionError
import pytest
import pandas.util.testing as tm

import pandas_datareader.data as web
from pandas_datareader.data import YahooDailyReader
from pandas_datareader._utils import RemoteDataError
from pandas_datareader._testing import skip_on_exception

XFAIL_REASON = 'Known connection failures on Yahoo when testing!'


class TestYahoo(object):

    @classmethod
    def setup_class(cls):
        pytest.importorskip("lxml")

    @skip_on_exception(RemoteDataError)
    def test_yahoo(self):
        # Asserts that yahoo is minimally working
        start = datetime(2010, 1, 1)
        end = datetime(2013, 1, 25)

        assert round(web.DataReader('F', 'yahoo', start, end)['Close'][-1],
                     2) == 13.68

    def test_yahoo_fails(self):
        start = datetime(2010, 1, 1)
        end = datetime(2013, 1, 27)

        with pytest.raises(Exception):
            web.DataReader('NON EXISTENT TICKER', 'yahoo', start, end)

    def test_get_quote_series(self):
        stringlist = ['GOOG', 'AAPL']
        fields = ['exchange', 'sharesOutstanding', 'epsForward']
        try:
            AAPL = web.get_quote_yahoo('AAPL')
            df = web.get_quote_yahoo(pd.Series(stringlist))
        except ConnectionError:
            pytest.xfail(reason=XFAIL_REASON)
        tm.assert_series_equal(AAPL.iloc[0][fields], df.loc['AAPL'][fields])
        assert sorted(stringlist) == sorted(list(df.index.values))

    def test_get_quote_string(self):
        try:
            df = web.get_quote_yahoo('GOOG')
        except ConnectionError:
            pytest.xfail(reason=XFAIL_REASON)

        assert not pd.isnull(df['marketCap'][0])

    def test_get_quote_stringlist(self):
        stringlist = ['GOOG', 'AAPL']
        try:
            df = web.get_quote_yahoo(stringlist)
        except ConnectionError:
            pytest.xfail(reason=XFAIL_REASON)
        assert sorted(stringlist) == sorted(list(df.index.values))

    def test_get_quote_comma_name(self):
        try:
            df = web.get_quote_yahoo(['RGLD'])
        except ConnectionError:
            pytest.xfail(reason=XFAIL_REASON)
        assert df['longName'][0] == 'Royal Gold, Inc.'

    @pytest.mark.skip('Unreliable test, receive partial '
                      'components back for dow_jones')
    def test_get_components_dow_jones(self):  # pragma: no cover
        df = web.get_components_yahoo('^DJI')  # Dow Jones
        assert isinstance(df, pd.DataFrame)
        assert len(df) == 30

    @pytest.mark.skip('Unreliable test, receive partial '
                      'components back for dax')
    def test_get_components_dax(self):  # pragma: no cover
        df = web.get_components_yahoo('^GDAXI')  # DAX
        assert isinstance(df, pd.DataFrame)

        assert len(df) == 30
        assert df[df.name.str.contains('adidas', case=False)].index == 'ADS.DE'

    @pytest.mark.skip('Unreliable test, receive partial '
                      'components back for nasdaq_100')
    def test_get_components_nasdaq_100(self):  # pragma: no cover
        # As of 7/12/13, the conditional will
        # return false because the link is invalid

        df = web.get_components_yahoo('^NDX')  # NASDAQ-100
        assert isinstance(df, pd.DataFrame)

        if len(df) > 1:
            # Usual culprits, should be around for a while
            assert 'AAPL' in df.index
            assert 'GOOG' in df.index
            assert 'AMZN' in df.index
        else:
            expected = DataFrame({'exchange': 'N/A', 'name': '@^NDX'},
                                 index=['@^NDX'])
            tm.assert_frame_equal(df, expected)

    @skip_on_exception(RemoteDataError)
    def test_get_data_single_symbol(self):
        # single symbol
        # http://finance.yahoo.com/q/hp?s=GOOG&a=09&b=08&c=2010&d=09&e=10&f=2010&g=d
        # just test that we succeed
        web.get_data_yahoo('GOOG')

    @skip_on_exception(RemoteDataError)
    def test_data_with_no_actions(self):
        web.get_data_yahoo('TSLA')

    @skip_on_exception(RemoteDataError)
    def test_get_data_adjust_price(self):
        goog = web.get_data_yahoo('GOOG')
        goog_adj = web.get_data_yahoo('GOOG', adjust_price=True)
        assert 'Adj Close' not in goog_adj.columns
        assert (goog['Open'] * goog_adj['Adj_Ratio']).equals(goog_adj['Open'])

    @skip_on_exception(RemoteDataError)
    def test_get_data_interval(self):
        # daily interval data
        pan = web.get_data_yahoo('XOM', '2013-01-01',
                                 '2013-12-31', interval='d')
        assert len(pan) == 252

        # weekly interval data
        pan = web.get_data_yahoo('XOM', '2013-01-01',
                                 '2013-12-31', interval='w')
        assert len(pan) == 53

        # monthly interval data
        pan = web.get_data_yahoo('XOM', '2012-12-31',
                                 '2013-12-31', interval='m')
        assert len(pan) == 12

        # test fail on invalid interval
        with pytest.raises(ValueError):
            web.get_data_yahoo('XOM', interval='NOT VALID')

    @skip_on_exception(RemoteDataError)
    def test_get_data_multiple_symbols(self):
        # just test that we succeed
        sl = ['AAPL', 'AMZN', 'GOOG']
        web.get_data_yahoo(sl, '2012')

    @pytest.mark.parametrize('adj_pr', [True, False])
    @skip_on_exception(RemoteDataError)
    def test_get_data_null_as_missing_data(self, adj_pr):
        result = web.get_data_yahoo('SRCE', '20160626', '20160705',
                                    adjust_price=adj_pr)
        # sanity checking
        floats = ['Open', 'High', 'Low', 'Close']
        if adj_pr:
            floats.append('Adj_Ratio')
        else:
            floats.append('Adj Close')

        assert result[floats].dtypes.all() == np.floating

    @skip_on_exception(RemoteDataError)
    def test_get_data_multiple_symbols_two_dates(self):
        data = web.get_data_yahoo(['GE', 'MSFT', 'INTC'], 'JAN-01-12',
                                  'JAN-31-12')
        result = data.Close.loc['01-18-12'].T
        assert result.size == 3

        # sanity checking
        assert result.dtypes == np.floating

        expected = np.array([[18.99, 28.4, 25.18],
                             [18.58, 28.31, 25.13],
                             [19.03, 28.16, 25.52],
                             [18.81, 28.82, 25.87]])
        df = data.Open
        result = df[(df.index >= 'Jan-15-12') & (df.index <= 'Jan-20-12')]
        assert expected.shape == result.shape

    def test_get_date_ret_index(self):
        pan = web.get_data_yahoo(['GE', 'INTC', 'IBM'], '1977', '1987',
                                 ret_index=True)
        assert hasattr(pan, 'Ret_Index')

        if hasattr(pan, 'Ret_Index') and hasattr(pan.Ret_Index, 'INTC'):
            tstamp = pan.Ret_Index.INTC.first_valid_index()
            result = pan.Ret_Index.loc[tstamp, 'INTC']
            assert result == 1.0

        # sanity checking
        assert np.issubdtype(pan.values.dtype, np.floating)

    def test_get_data_yahoo_actions(self):
        start = datetime(1990, 1, 1)
        end = datetime(2000, 4, 5)

        actions = web.get_data_yahoo_actions('BHP.AX', start, end,
                                             adjust_dividends=True)

        assert sum(actions['action'] == 'DIVIDEND') == 21
        assert sum(actions['action'] == 'SPLIT') == 1

        assert actions.loc['1995-05-11', 'action'][0] == 'SPLIT'
        assert actions.loc['1995-05-11', 'value'][0] == 1 / 1.1

        assert actions.loc['1993-05-10', 'action'][0] == 'DIVIDEND'
        assert actions.loc['1993-05-10', 'value'][0] == 0.21

    def test_get_data_yahoo_actions_invalid_symbol(self):
        start = datetime(1990, 1, 1)
        end = datetime(2000, 4, 5)

        with pytest.raises(IOError):
            web.get_data_yahoo_actions('UNKNOWN TICKER', start, end)

    @skip_on_exception(RemoteDataError)
    def test_yahoo_reader_class(self):
        r = YahooDailyReader('GOOG')
        df = r.read()

        assert df.Volume.loc['JAN-02-2015'] == 1447600

        session = requests.Session()

        r = YahooDailyReader('GOOG', session=session)
        assert r.session is session

    def test_yahoo_DataReader(self):
        start = datetime(2010, 1, 1)
        end = datetime(2015, 5, 9)
        result = web.DataReader('AAPL', 'yahoo-actions', start, end)

        exp_idx = pd.DatetimeIndex(['2015-05-07', '2015-02-05',
                                    '2014-11-06', '2014-08-07',
                                    '2014-06-09', '2014-05-08',
                                    '2014-02-06', '2013-11-06',
                                    '2013-08-08', '2013-05-09',
                                    '2013-02-07', '2012-11-07',
                                    '2012-08-09'])

        exp = pd.DataFrame({'action': ['DIVIDEND', 'DIVIDEND', 'DIVIDEND',
                                       'DIVIDEND', 'SPLIT', 'DIVIDEND',
                                       'DIVIDEND', 'DIVIDEND',
                                       'DIVIDEND', 'DIVIDEND', 'DIVIDEND',
                                       'DIVIDEND', 'DIVIDEND'],
                            'value': [0.52, 0.47, 0.47, 0.47, 0.14285714,
                                      3.29, 3.05, 3.05, 3.05,
                                      3.05, 2.65, 2.65, 2.65]},
                           index=exp_idx)
        exp.index.name = 'Date'
        tm.assert_frame_equal(result.reindex_like(exp).round(2), exp.round(2))

        result = web.get_data_yahoo_actions('AAPL', start, end,
                                            adjust_dividends=True)

        exp = pd.DataFrame({'action': ['DIVIDEND', 'DIVIDEND', 'DIVIDEND',
                                       'DIVIDEND', 'SPLIT', 'DIVIDEND',
                                       'DIVIDEND', 'DIVIDEND',
                                       'DIVIDEND', 'DIVIDEND', 'DIVIDEND',
                                       'DIVIDEND', 'DIVIDEND'],
                            'value': [0.52, 0.47, 0.47, 0.47, 0.14285714,
                                      0.47, 0.43571, 0.43571, 0.43571,
                                      0.43571, 0.37857, 0.37857, 0.37857]},
                           index=exp_idx)
        exp.index.name = 'Date'
        tm.assert_frame_equal(result.reindex_like(exp).round(5), exp.round(5))

    @skip_on_exception(RemoteDataError)
    def test_yahoo_DataReader_multi(self):
        start = datetime(2010, 1, 1)
        end = datetime(2015, 5, 9)
        result = web.DataReader(['AAPL', 'F'], 'yahoo-actions', start, end)
        assert isinstance(result, dict)



from pandas_datareader.iex import IEX

# Data provided for free by IEX
# Data is furnished in compliance with the guidelines promulgated in the IEX
# API terms of service and manual
# See https://iextrading.com/api-exhibit-a/ for additional information
# and conditions of use


class MarketReader(IEX):
    """
    Near real-time traded volume

    Notes
    -----
    Market data is captured by the IEX system between approximately 7:45 a.m.
    and 5:15 p.m. ET.
    """
    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None):
        super(MarketReader, self).__init__(symbols=symbols,
                                           start=start, end=end,
                                           retry_count=retry_count,
                                           pause=pause, session=session)

    @property
    def service(self):
        """Service endpoint"""
        return "market"

    def _get_params(self, symbols):
        # Market API does not take any parameters, returning empty dict
        return {}



import os
import pytest

import pandas_datareader.data as web
import pandas as pd

import numpy as np

TEST_API_KEY = os.getenv('ALPHAVANTAGE_API_KEY')
TEST_API_KEY = None if not TEST_API_KEY else TEST_API_KEY


class TestAVQuotes(object):

    @classmethod
    def setup_class(cls):
        pytest.importorskip("lxml")

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_invalid_symbol(self):
        with pytest.raises(ValueError):
            web.get_quote_av("BADSYMBOL")

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_bad_multiple_symbol(self):
        with pytest.raises(ValueError):
            web.get_quote_av(["AAPL", "BADSYMBOL"])

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_single_symbol(self):
        df = web.get_quote_av("AAPL")
        assert len(df) == 1

        expected = pd.Index(["price", "volume", "timestamp"])
        assert df.columns.equals(expected)

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    def test_multi_symbol(self):
        df = web.get_quote_av(["AAPL", "TSLA"])
        assert len(df) == 2

        expected = pd.Index(["price", "volume", "timestamp"])
        assert df.columns.equals(expected)

    @pytest.mark.skipif(TEST_API_KEY is None,
                        reason="ALPHAVANTAGE_API_KEY not set")
    @pytest.mark.xfail(reason="May return NaN outside of market hours")
    def test_return_types(self):
        df = web.get_quote_av("AAPL")

        assert isinstance(df["AAPL"]["price"], np.int64)
        assert isinstance(df["AAPL"]["volume"], np.float64)



from pandas_datareader.iex import IEX

# Data provided for free by IEX
# Data is furnished in compliance with the guidelines promulgated in the IEX
# API terms of service and manual
# See https://iextrading.com/api-exhibit-a/ for additional information
# and conditions of use


class SymbolsReader(IEX):
    """
    Symbols available for trading on IEX

    Notes
    -----
    Returns symbols IEX supports for trading. Updated daily as of 7:45 a.m.
    ET.
    """
    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None):
        super(SymbolsReader, self).__init__(symbols=symbols,
                                            start=start, end=end,
                                            retry_count=retry_count,
                                            pause=pause, session=session)

    @property
    def service(self):
        """Service endpoint"""
        return "ref-data/symbols"

    def _get_params(self, symbols):
        # Ref Data API does not take any parameters, returning empty dict
        return {}



import time
from datetime import datetime
from warnings import warn

import requests
from pandas import DataFrame
import numpy as np

from pandas_datareader._utils import SymbolWarning
from pandas_datareader.base import _BaseReader
from pandas_datareader.exceptions import ImmediateDeprecationError, \
    DEP_ERROR_MSG
import pandas as pd


class MorningstarDailyReader(_BaseReader):
    """
    Read daily data from Morningstar

    Parameters
    ----------
    symbols : {str, List[str]}
        String symbol of like of symbols
    start : string, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : string, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : float, default 0.1
        Time, in seconds, of the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    freq : {str, None}
        Frequency to use in select readers
    incl_splits : bool, optional
        Include splits in data
    incl_dividends : bool,, optional
        Include divdends in data
    incl_volume : bool, optional
        Include volume in data
    currency : str, optional
        Currency to use for data
    interval : str, optional
        Sampling interval to use for downloaded data

    Notes
    -----
    See `Morningstar <http://www.morningstar.com>`__
    """

    def __init__(self, symbols, start=None, end=None, retry_count=3,
                 pause=0.1, timeout=30, session=None, freq=None,
                 incl_splits=False, incl_dividends=False, incl_volume=True,
                 currency='usd', interval='d'):
        raise ImmediateDeprecationError(DEP_ERROR_MSG.format("Morningstar"))
        super(MorningstarDailyReader, self).__init__(symbols, start, end,
                                                     retry_count, pause,
                                                     timeout, session, freq)

        self.incl_splits = incl_splits
        self.incl_dividends = incl_dividends
        self.incl_vol = incl_volume
        self.currency = currency
        self.interval = interval

    def _url_params(self):
        if self.interval not in ['d', 'wk', 'mo', 'm', 'w']:
            raise ValueError("Invalid interval: valid values are  'd', 'wk' "
                             "and 'mo'. 'm' and 'w' have been implemented for "
                             "backward compatibility")
        elif self.interval in ['m', 'mo']:
            self.interval = 'm'
        elif self.interval in ['w', 'wk']:
            self.interval = 'w'

        if self.currency != "usd":
            warn("Caution! There is no explicit check for a valid currency "
                 "acronym\nIf an error is encountered consider changing this "
                 "value.")

        p = {"range": "|".join(
            [self.start.strftime("%Y-%m-%d"), self.end.strftime("%Y-%m-%d")]),
            "f": self.interval, "curry": self.currency,
            "dtype": "his", "showVol": "true",
            "hasF": "true", "isD": "true", "isS": "true",
            "ProdCode": "DIRECT"}

        return p

    @property
    def url(self):
        """API URL"""
        return "http://globalquote.morningstar.com/globalcomponent/" \
               "RealtimeHistoricalStockData.ashx"

    def _get_crumb(self, *args):
        """Not required """
        pass

    def _dl_mult_symbols(self, symbols, symbol_data=None):
        failed = []
        if symbol_data is None:
            symbol_data = []
        for symbol in symbols:
            params = self._url_params()
            params.update({"ticker": symbol})

            try:
                resp = requests.get(self.url, params=params)
            except (requests.HTTPError, requests.ConnectionError,
                    requests.RequestException):
                if symbol not in failed:
                    if self.retry_count == 0:
                        warn("skipping symbol %s: number of retries "
                             "exceeded." % symbol)
                        pass
                    else:
                        print("adding %s to retry list" % symbol)
                        failed.append(symbol)
            else:
                if resp.status_code == requests.codes.ok:
                    jsondata = resp.json()
                    if jsondata is None:
                        failed.append(symbol)
                        pass
                    else:
                        jsdata = self._restruct_json(symbol=symbol,
                                                     jsondata=jsondata)
                        symbol_data.extend(jsdata)
                else:
                    raise Exception("Request Error!: %s : %s" % (
                        resp.status_code, resp.reason))

            time.sleep(self.pause)

        if len(failed) > 0 and self.retry_count > 0:
            self.retry_count -= 1
            self._dl_mult_symbols(symbols=failed, symbol_data=symbol_data)
        else:
            self.retry_count = 0

        if len(symbol_data) == 0 and self.retry_count == 0:
            raise ValueError('All symbols were invalid')
        elif self.retry_count == 0 and len(failed) > 0:
            warn("The following symbols were excluded do to http "
                 "request errors: \n %s" % failed, SymbolWarning)

        symbols_df = DataFrame(data=symbol_data)
        dfx = symbols_df.set_index(["Symbol", "Date"])
        return dfx

    @staticmethod
    def _convert_index2date(indexvals):
        base = pd.to_datetime('1900-1-1')
        return [base + pd.to_timedelta(iv, unit='d') for iv in indexvals]

    def _restruct_json(self, symbol, jsondata):

        divdata = jsondata["DividendData"]

        pricedata = jsondata["PriceDataList"][0]["Datapoints"]
        dateidx = jsondata["PriceDataList"][0]["DateIndexs"]
        if jsondata["VolumeList"]:
            volumes = jsondata["VolumeList"]["Datapoints"]
        else:
            volumes = None

        dates = self._convert_index2date(indexvals=dateidx)
        barss = []
        for p in range(len(pricedata)):
            bar = pricedata[p]
            d = dates[p]
            bardict = {
                "Symbol": symbol, "Date": d, "Close": bar[0], "High": bar[1],
                "Low": bar[2], "Open": bar[3]}
            if len(divdata) == 0:
                pass
            else:
                events = []
                for x in divdata:
                    delta = (datetime.strptime(x["Date"], "%Y-%m-%d")
                             - d.to_pydatetime())
                    if delta.days == 0:
                        events.append(x)
                for e in events:
                    if self.incl_dividends and e["Type"].find("Div") > -1:
                        val = e["Desc"].replace(e["Type"], "")
                        bardict.update({"isDividend": val})
                    elif (self.incl_splits is True and
                          e["Type"].find("Split") > -1):
                        val = e["Desc"].replace(e["Type"], "")
                        bardict.update({"isSplit": val})
                    else:
                        pass
            if self.incl_vol is True:
                if volumes is None:
                    bardict.update({"Volume": np.nan})
                else:
                    bardict.update({"Volume": int(volumes[p] * 1000000)})
            else:
                pass

            barss.append(bardict)
        return barss

    def read(self):
        """Read data"""
        if isinstance(self.symbols, str):
            symbols = [self.symbols]
        else:
            symbols = self.symbols

        is_str = False
        try:
            is_str = all(map(lambda v: isinstance(v, str), symbols))
        except ValueError:
            pass

        if not is_str:
            raise TypeError("symbols must be iterable or string and not "
                            "type %s" % type(self.symbols))

        df = self._dl_mult_symbols(symbols=symbols)
        if len(df.index.levels[0]) == 0:
            raise ValueError("None of the provided symbols were valid")
        else:
            return df



import time

import numpy as np
import pandas as pd
import pandas.util.testing as tm
import pytest
import requests

from pandas_datareader.compat import assert_raises_regex
from pandas_datareader.wb import (search, download, get_countries,
                                  get_indicators, WorldBankReader)
from pandas_datareader._utils import RemoteDataError
from pandas_datareader._testing import skip_on_exception


class TestWB(object):

    def test_wdi_search(self):

        # Test that a name column exists, and that some results were returned
        # ...without being too strict about what the actual contents of the
        # results actually are.  The fact that there are some, is good enough.

        result = search('gdp.*capita.*constant')
        assert result.name.str.contains('GDP').any()

        # check cache returns the results within 0.5 sec
        current_time = time.time()
        result = search('gdp.*capita.*constant')
        assert result.name.str.contains('GDP').any()
        assert time.time() - current_time < 0.5

        result2 = WorldBankReader().search('gdp.*capita.*constant')
        session = requests.Session()

        result3 = search('gdp.*capita.*constant', session=session)
        result4 = WorldBankReader(session=session).search(
            'gdp.*capita.*constant')

        for result in [result2, result3, result4]:
            assert result.name.str.contains('GDP').any()

    def test_wdi_download(self):

        # Test a bad indicator with double (US), triple (USA),
        # standard (CA, MX), non standard (KSV),
        # duplicated (US, US, USA), and unknown (BLA) country codes

        # ...but NOT a crash inducing country code (World Bank strips pandas
        #    users of the luxury of laziness because they create their
        #    own exceptions, and don't clean up legacy country codes.
        # ...but NOT a retired indicator (user should want it to error).

        cntry_codes = ['CA', 'MX', 'USA', 'US', 'US', 'KSV', 'BLA']
        inds = ['NY.GDP.PCAP.CD', 'BAD.INDICATOR']

        # These are the expected results, rounded (robust against
        # data revisions in the future).
        expected = {'NY.GDP.PCAP.CD': {('Canada', '2004'): 32000.0,
                                       ('Canada', '2003'): 28000.0,
                                       ('Kosovo', '2004'): 2000.0,
                                       ('Kosovo', '2003'): 2000.0,
                                       ('Mexico', '2004'): 7000.0,
                                       ('Mexico', '2003'): 7000.0,
                                       ('United States', '2004'): 42000.0,
                                       ('United States', '2003'): 40000.0}}
        expected = pd.DataFrame(expected)
        expected = expected.sort_index()

        result = download(country=cntry_codes, indicator=inds,
                          start=2003, end=2004, errors='ignore')
        result = result.sort_index()

        # Round, to ignore revisions to data.
        result = np.round(result, decimals=-3)

        expected.index.names = ['country', 'year']
        tm.assert_frame_equal(result, expected)

        # pass start and end as string
        result = download(country=cntry_codes, indicator=inds,
                          start='2003', end='2004', errors='ignore')
        result = result.sort_index()

        # Round, to ignore revisions to data.
        result = np.round(result, decimals=-3)
        tm.assert_frame_equal(result, expected)

    def test_wdi_download_str(self):

        # These are the expected results, rounded (robust against
        # data revisions in the future).
        expected = {'NY.GDP.PCAP.CD': {('Japan', '2004'): 38000.0,
                                       ('Japan', '2003'): 35000.0,
                                       ('Japan', '2002'): 32000.0,
                                       ('Japan', '2001'): 34000.0,
                                       ('Japan', '2000'): 39000.0}}
        expected = pd.DataFrame(expected)
        expected = expected.sort_index()

        cntry_codes = 'JP'
        inds = 'NY.GDP.PCAP.CD'
        result = download(country=cntry_codes, indicator=inds,
                          start=2000, end=2004, errors='ignore')
        result = result.sort_index()
        result = np.round(result, decimals=-3)

        expected.index.names = ['country', 'year']
        tm.assert_frame_equal(result, expected)

        result = WorldBankReader(inds, countries=cntry_codes,
                                 start=2000, end=2004, errors='ignore').read()
        result = result.sort_index()
        result = np.round(result, decimals=-3)
        tm.assert_frame_equal(result, expected)

    def test_wdi_download_error_handling(self):
        cntry_codes = ['USA', 'XX']
        inds = 'NY.GDP.PCAP.CD'

        msg = "Invalid Country Code\\(s\\): XX"
        with assert_raises_regex(ValueError, msg):
            download(country=cntry_codes, indicator=inds,
                     start=2003, end=2004, errors='raise')

        with tm.assert_produces_warning():
            result = download(country=cntry_codes, indicator=inds,
                              start=2003, end=2004, errors='warn')
            assert isinstance(result, pd.DataFrame)
            assert len(result), 2

        cntry_codes = ['USA']
        inds = ['NY.GDP.PCAP.CD', 'BAD_INDICATOR']

        msg = ("The provided parameter value is not valid\\. "
               "Indicator: BAD_INDICATOR")
        with assert_raises_regex(ValueError, msg):
            download(country=cntry_codes, indicator=inds,
                     start=2003, end=2004, errors='raise')

        with tm.assert_produces_warning():
            result = download(country=cntry_codes, indicator=inds,
                              start=2003, end=2004, errors='warn')
            assert isinstance(result, pd.DataFrame)
            assert len(result) == 2

    def test_wdi_download_w_retired_indicator(self):

        cntry_codes = ['CA', 'MX', 'US']
        # Despite showing up in the search feature, and being listed online,
        # the api calls to GDPPCKD don't work in their own query builder, nor
        # pandas module.  GDPPCKD used to be a common symbol.
        #
        # This test is written to ensure that error messages to pandas users
        # continue to make sense, rather than a user getting some missing
        # key error, cause their JSON message format changed.  If
        #
        # World bank ever finishes the deprecation of this symbol,
        # this test should still pass.

        inds = ['GDPPCKD']

        with pytest.raises(ValueError):
            result = download(country=cntry_codes, indicator=inds,
                              start=2003, end=2004, errors='ignore')

            # If it ever gets here, it means WB unretired the indicator.
            # even if they dropped it completely, it would still
            # get caught above
            # or the WB API changed somehow in a really
            # unexpected way.
            if len(result) > 0:  # pragma: no cover
                pytest.skip("Invalid results")

    def test_wdi_download_w_crash_inducing_countrycode(self):

        cntry_codes = ['CA', 'MX', 'US', 'XXX']
        inds = ['NY.GDP.PCAP.CD']

        with pytest.raises(ValueError):
            result = download(country=cntry_codes, indicator=inds,
                              start=2003, end=2004, errors='ignore')

            # If it ever gets here, it means the country code XXX
            # got used by WB
            # or the WB API changed somehow in a really
            # unexpected way.
            if len(result) > 0:  # pragma: no cover
                pytest.skip("Invalid results")

    def test_wdi_get_countries(self):
        result1 = get_countries()
        result2 = WorldBankReader().get_countries()

        session = requests.Session()
        result3 = get_countries(session=session)
        result4 = WorldBankReader(session=session).get_countries()

        for result in [result1, result2, result3, result4]:
            assert 'Zimbabwe' in list(result['name'])
            assert len(result) > 100
            assert pd.notnull(result.latitude.mean())
            assert pd.notnull(result.longitude.mean())

    def test_wdi_get_indicators(self):
        result1 = get_indicators()
        result2 = WorldBankReader().get_indicators()

        session = requests.Session()
        result3 = get_indicators(session=session)
        result4 = WorldBankReader(session=session).get_indicators()

        for result in [result1, result2, result3, result4]:
            exp_col = pd.Index(['id', 'name', 'source', 'sourceNote',
                                'sourceOrganization', 'topics', 'unit'])
            # assert_index_equal doesn't exists
            assert result.columns.equals(exp_col)
            assert len(result) > 10000

    @skip_on_exception(RemoteDataError)
    def test_wdi_download_monthly(self):
        expected = {'COPPER': {('World', '2012M01'): 8040.47,
                               ('World', '2011M12'): 7565.48,
                               ('World', '2011M11'): 7581.02,
                               ('World', '2011M10'): 7394.19,
                               ('World', '2011M09'): 8300.14,
                               ('World', '2011M08'): 9000.76,
                               ('World', '2011M07'): 9650.46,
                               ('World', '2011M06'): 9066.85,
                               ('World', '2011M05'): 8959.90,
                               ('World', '2011M04'): 9492.79,
                               ('World', '2011M03'): 9503.36,
                               ('World', '2011M02'): 9867.60,
                               ('World', '2011M01'): 9555.70}}
        expected = pd.DataFrame(expected)
        # Round, to ignore revisions to data.
        expected = np.round(expected, decimals=-3)
        expected = expected.sort_index()
        cntry_codes = 'ALL'
        inds = 'COPPER'
        result = download(country=cntry_codes, indicator=inds,
                          start=2011, end=2012, freq='M', errors='ignore')
        result = result.sort_index()
        result = np.round(result, decimals=-3)

        expected.index.names = ['country', 'year']
        tm.assert_frame_equal(result, expected)

        result = WorldBankReader(inds, countries=cntry_codes, start=2011,
                                 end=2012, freq='M', errors='ignore').read()
        result = result.sort_index()
        result = np.round(result, decimals=-3)
        tm.assert_frame_equal(result, expected)

    def test_wdi_download_quarterly(self):
        code = 'DT.DOD.PUBS.CD.US'
        expected = {code: {('Albania', '2012Q1'): 3240539817.18,
                           ('Albania', '2011Q4'): 3213979715.15,
                           ('Albania', '2011Q3'): 3187681048.95,
                           ('Albania', '2011Q2'): 3248041513.86,
                           ('Albania', '2011Q1'): 3137210567.92}}
        expected = pd.DataFrame(expected)
        # Round, to ignore revisions to data.
        expected = np.round(expected, decimals=-3)
        expected = expected.sort_index()
        cntry_codes = 'ALB'
        inds = 'DT.DOD.PUBS.CD.US'
        result = download(country=cntry_codes, indicator=inds,
                          start=2011, end=2012, freq='Q', errors='ignore')
        result = result.sort_index()
        result = np.round(result, decimals=-3)

        expected.index.names = ['country', 'year']
        tm.assert_frame_equal(result, expected)

        result = WorldBankReader(inds, countries=cntry_codes, start=2011,
                                 end=2012, freq='Q', errors='ignore').read()
        result = result.sort_index()
        result = np.round(result, decimals=-1)
        tm.assert_frame_equal(result, expected)



from pandas_datareader.base import _DailyBaseReader
from pandas_datareader.exceptions import ImmediateDeprecationError, \
    DEP_ERROR_MSG


class GoogleDailyReader(_DailyBaseReader):
    """
    Returns DataFrame of historical stock prices from
    symbols, over date range, start to end. To avoid being penalized by Google
    Finance servers, pauses between downloading 'chunks' of symbols can be
    specified.

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    start : string, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : string, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        Number of symbols to download consecutively before intiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used
    """

    def __init__(self, symbols=None, start=None, end=None, retry_count=3,
                 pause=0.1, session=None, chunksize=25):
        raise ImmediateDeprecationError(DEP_ERROR_MSG.format('Google finance'))
        super(GoogleDailyReader, self).__init__(symbols, start, end,
                                                retry_count, pause, session,
                                                chunksize)

    @property
    def url(self):
        """API URL"""
        # Thanks user:vnmabus for pointing this out.
        return 'https://finance.google.co.uk/bctzjpnsun/historical'

    def _get_params(self, symbol):
        params = {
            'q': symbol,
            'startdate': self.start.strftime('%b %d, %Y'),
            'enddate': self.end.strftime('%b %d, %Y'),
            'output': "csv"
        }
        return params



# pylint: disable-msg=E1101,W0613,W0603

import os

import numpy as np
import pandas as pd
import pandas.util.testing as tm
import pytest

from pandas_datareader.compat import PANDAS_0210
from pandas_datareader.io import read_jsdmx


@pytest.fixture
def dirpath(datapath):
    return datapath("io", "data")


@pytest.mark.skipif(not PANDAS_0210, reason='Broken on old pandas')
def test_tourism(dirpath):
    # OECD -> Industry and Services -> Inbound Tourism
    result = read_jsdmx(os.path.join(dirpath, 'jsdmx',
                                     'tourism.json'))
    assert isinstance(result, pd.DataFrame)
    jp = result['Japan']
    visitors = ['China', 'Hong Kong, China',
                'Total international arrivals',
                'Korea', 'Chinese Taipei', 'United States']

    exp_col = pd.Index(
        ['China', 'Hong Kong, China', 'Total international arrivals',
         'Korea', 'Chinese Taipei', 'United States'],
        name='Variable')
    exp_idx = pd.DatetimeIndex(['2008-01-01', '2009-01-01', '2010-01-01',
                                '2011-01-01', '2012-01-01', '2013-01-01',
                                '2014-01-01', '2015-01-01', '2016-01-01'],
                               name='Year')
    values = [
        [1000000.0, 550000.0, 8351000.0, 2382000.0, 1390000.0, 768000.0],
        [1006000.0, 450000.0, 6790000.0, 1587000.0, 1024000.0, 700000.0],
        [1413000.0, 509000.0, 8611000.0, 2440000.0, 1268000.0, 727000.0],
        [1043000.0, 365000.0, 6219000.0, 1658000.0, 994000.0, 566000.0],
        [1430000.0, 482000.0, 8368000.0, 2044000.0, 1467000.0, 717000.0],
        [1314000.0, 746000.0, 10364000.0, 2456000.0, 2211000.0, 799000.0],
        [2409000.0, 926000.0, 13413000.0, 2755000.0, 2830000.0, 892000.0],
        [4993689.0, 1524292.0, 19737409.0, 4002095.0, 3677075.0,
         1033258.0],
        [6373564.0, 1839193.0, 24039700.0, 5090302.0, 4167512.0, 1242719.0]
    ]
    values = np.array(values, dtype='object')
    expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
    tm.assert_frame_equal(jp[visitors], expected)


@pytest.mark.skipif(not PANDAS_0210, reason='Broken on old pandas')
def test_land_use(dirpath):
    # OECD -> Environment -> Resources Land Use
    result = read_jsdmx(os.path.join(dirpath, 'jsdmx',
                                     'land_use.json'))
    assert isinstance(result, pd.DataFrame)
    result = result.loc['2010':'2011']

    cols = ['Arable land and permanent crops',
            'Arable and cropland % land area',
            'Total area', 'Forest', 'Forest % land area',
            'Land area', 'Permanent meadows and pastures',
            'Meadows and pastures % land area', 'Other areas',
            'Other % land area']
    exp_col = pd.MultiIndex.from_product([
        ['Japan', 'United States'],
        cols], names=['Country', 'Variable'])
    exp_idx = pd.DatetimeIndex(['2010', '2011'], name='Year')
    values = [
        [53790.0, 14.753154141525, 377800.0, np.nan, np.nan, 364600.0,
         5000.0, 1.3713658804169, np.nan, np.nan,
         1897990.0, 20.722767650476, 9629090.0, np.nan, np.nan, 9158960.0,
         2416000.0, 26.378540795025, np.nan,
         np.nan],
        [53580.0, 14.691527282698, 377800.0, np.nan, np.nan, 364700.0,
         5000.0, 1.3709898546751, np.nan, np.nan,
         1897990.0, 20.722767650476, 9629090.0, np.nan, np.nan, 9158960.0,
         2416000.0, 26.378540795025, np.nan,
         np.nan]]
    values = np.array(values)
    expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
    tm.assert_frame_equal(result[exp_col], expected)



from datetime import datetime

import pytest

import numpy as np
import pandas as pd
import pandas.util.testing as tm
import pandas_datareader.data as web

from pandas import DataFrame
from pandas_datareader._utils import RemoteDataError


class TestFred(object):

    def test_fred(self):

        # Raises an exception when DataReader can't
        # get a 200 response from FRED.

        start = datetime(2010, 1, 1)
        end = datetime(2013, 1, 1)

        df = web.DataReader("GDP", "fred", start, end)
        ts = df['GDP']

        assert ts.index[0] == pd.to_datetime("2010-01-01")
        assert ts.index[-1] == pd.to_datetime("2013-01-01")
        assert ts.index.name == "DATE"
        assert ts.name == "GDP"
        assert len(ts) == 13

        with pytest.raises(RemoteDataError):
            web.DataReader("NON EXISTENT SERIES", 'fred', start, end)

    def test_fred_nan(self):
        start = datetime(2010, 1, 1)
        end = datetime(2013, 1, 27)
        df = web.DataReader("DFII5", "fred", start, end)
        assert pd.isnull(df.loc['2010-01-01'][0])

    def test_fred_parts(self):  # pragma: no cover
        start = datetime(2010, 1, 1)
        end = datetime(2013, 1, 27)
        df = web.get_data_fred("CPIAUCSL", start, end)
        assert df.loc['2010-05-01'][0] == 217.29

        t = df.CPIAUCSL.values
        assert np.issubdtype(t.dtype, np.floating)
        assert t.shape == (37,)

    def test_fred_part2(self):
        expected = [[576.7],
                    [962.9],
                    [684.7],
                    [848.3],
                    [933.3]]
        result = web.get_data_fred("A09024USA144NNBR", start="1915").iloc[:5]
        tm.assert_numpy_array_equal(result.values, np.array(expected))

    def test_invalid_series(self):
        name = "NOT A REAL SERIES"
        with pytest.raises(Exception):
            web.get_data_fred(name)

    def test_fred_multi(self):  # pragma: no cover
        names = ['CPIAUCSL', 'CPALTT01USQ661S', 'CPILFESL']
        start = datetime(2010, 1, 1)
        end = datetime(2013, 1, 27)

        received = web.DataReader(names, "fred", start, end).head(1)

        expected = DataFrame([[217.488, 99.68746, 220.633]], columns=names,
                             index=[pd.Timestamp('2010-01-01 00:00:00')])
        expected.index.rename('DATE', inplace=True)
        tm.assert_frame_equal(received, expected, check_less_precise=True)

    def test_fred_multi_bad_series(self):
        names = ['NOTAREALSERIES', 'CPIAUCSL', "ALSO FAKE"]
        with pytest.raises(RemoteDataError):
            web.DataReader(names, data_source="fred")
